/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "chart.h"
#include "memory.h"
#include "main.h"

Graph *ActiveGraph;

static int nchart = 1;

Graph *
newchart ()
{
  int i;
  Graph *chart;
  chart = (Graph *) absmalloc (sizeof (Graph), "newchart:chart ");

  chart->zm = 100;
  chart->numrange = 0;
  chart->type = 0;
  chart->graphic_grid = grboth;
  chart->axe_cat_grid = 0;
  chart->axe_val_grid = 0;

  chart->Axes[xlCategory].HasMajorGridLines = 0;
  chart->Axes[xlCategory].MinimumScaleIsAuto = True;
  chart->Axes[xlCategory].MaximumScaleIsAuto = True;
  chart->Axes[xlCategory].MajorUnitIsAuto = True;
  chart->Axes[xlCategory].HasTitle = True;
  title_settext (&chart->Axes[xlCategory].AxisTitle, "'X axis");

  chart->Axes[xlValue].HasMajorGridLines = 0;
  chart->Axes[xlValue].MinimumScaleIsAuto = True;
  chart->Axes[xlValue].MaximumScaleIsAuto = True;
  chart->Axes[xlValue].MajorUnitIsAuto = True;
  chart->Axes[xlValue].HasTitle = True;
  title_settext (&chart->Axes[xlValue].AxisTitle, "'Y axis");

  for (i = 0; i < 10; i++)
    {
      sprintf ((chart->range[i]).name, "'Data%d", i);
      chart->graphic_format[i] = grboth;
      chart->SeriesCollections[i].gr = chart;
      chart->SeriesCollections[i].num = i;
    }

  chart->HasTitle = True;
  title_settext (&chart->ChartTitle, "'title");

  sprintf ((chart->range[0]).name, "'X axis");

  chart->objn = nchart++;

  return chart;
};

Graph *
newchart1 (gr)
     Graph *gr;
{
  Graph *chart = newchart ();
  chart_copy (chart, gr);
  chart->objn = nchart++;
  return chart;
};

int
chart_copy (chart, gr)
     Graph *chart;
     Graph *gr;
{
  int i;
  if (chart == NULL || gr == NULL || (gr == chart))
    return 1;

  chart->zm = gr->zm;
  chart->numrange = gr->numrange;
  chart->type = gr->type;
  chart->graphic_grid = gr->graphic_grid;
  chart->axe_cat_grid = gr->axe_cat_grid;
  chart->Axes[xlCategory].HasMajorGridLines = gr->Axes[xlCategory].HasMajorGridLines;
  chart->Axes[xlCategory].MinimumScaleIsAuto = gr->Axes[xlCategory].MinimumScaleIsAuto;
  chart->Axes[xlCategory].MajorUnitIsAuto = gr->Axes[xlCategory].MajorUnitIsAuto;
  chart->Axes[xlCategory].HasTitle = gr->Axes[xlCategory].HasTitle;

  chart->Axes[xlValue].HasMajorGridLines = gr->Axes[xlValue].HasMajorGridLines;
  chart->Axes[xlValue].MinimumScaleIsAuto = gr->Axes[xlValue].MinimumScaleIsAuto;
  chart->Axes[xlValue].MajorUnitIsAuto = gr->Axes[xlValue].MajorUnitIsAuto;
  chart->Axes[xlValue].HasTitle = gr->Axes[xlValue].HasTitle;

  for (i = 0; i < 10; i++)
    {
      strcpy ((chart->range[i]).name, (gr->range[i]).name);
      chart->graphic_format[i] = gr->graphic_format[i];
      chart->SeriesCollections[i].gr = (char *) chart;
      chart->SeriesCollections[i].num = i;
    }

  chart->HasTitle = gr->HasTitle;

  return 0;
}

int
freechart (gr)
     Graph *gr;
{
  if (gr == NULL)
    return -1;
  title_free (&gr->ChartTitle);
  title_free (&gr->Axes[xlCategory].AxisTitle);
  title_free (&gr->Axes[xlValue].AxisTitle);
  absfree (gr, "freechart:gr ");
  return 0;
};

int
chart_w (gr, fp)
     Graph *gr;
     FILE *fp;
{
  char i1j1[16];
  char i2j2[16];
  int j;
  char *type;
  switch (gr->type)
    {
    case 0:
      type = "xlLine";
      break;
    case 1:
      type = "xlPie";
      break;
    case 2:
      type = "xlbarClustered";
      break;
    }

  fprintf (fp, "Set objchart = ActiveWorksheet.ChartObjects.Add (%d,%d,%d,%d)\n", gr->x, gr->y, gr->w, gr->h);
  fprintf (fp, "objchart.Activate\n");
  fprintf (fp, "ActiveChart.ChartType = %s\n", type);

  fprintf (fp, "ActiveChart.SeriesCollection.NewSerie\n");
  n2a (i1j1, (gr->range[0]).i1, (gr->range[0]).j1);
  n2a (i2j2, (gr->range[0]).i2, (gr->range[0]).j2);
  fprintf (fp, "ActiveChart.SeriesCollection(1).XValues = Range(\"%s:%s\")\n", i1j1, i2j2);

  for (j = 1; j < gr->numrange; j++)
    {
      n2a (i1j1, (gr->range[j]).i1, (gr->range[j]).j1);
      n2a (i2j2, (gr->range[j]).i2, (gr->range[j]).j2);
      fprintf (fp, "ActiveChart.SeriesCollection(%d).Name = \"%s\"\n", j, (gr->range[j]).name);
      fprintf (fp, "ActiveChart.SeriesCollection(%d).Values = Range(\"%s:%s\")\n", j, i1j1, i2j2);
    }

  if (gr->HasTitle)
    {
      fprintf (fp, "ActiveChart.HasTitle = True\n");
      fprintf (fp, "ActiveChart.ChartTitle.Text = \"%s\"\n", gr->ChartTitle.Text);
    }

  if (gr->Axes[xlCategory].HasMajorGridLines)
    fprintf (fp, "ActiveChart.Axes(xlCategory).HasMajorGridlines = True\n");

  if (gr->Axes[xlCategory].HasTitle)
    {
      fprintf (fp, "ActiveChart.Axes(xlCategory).HasTitle = True\n");
      fprintf (fp, "ActiveChart.Axes(xlCategory).AxisTitle.Text = \"%s\"\n",
	       gr->Axes[xlCategory].AxisTitle.Text);
    }

  if (gr->Axes[xlValue].HasMajorGridLines)
    fprintf (fp, "ActiveChart.Axes(xlValue).HasMajorGridlines = True\n");

  if (gr->Axes[xlValue].HasTitle)
    {
      fprintf (fp, "ActiveChart.Axes(xlValue   ).HasTitle = True\n");
      fprintf (fp, "ActiveChart.Axes(xlValue   ).AxisTitle.Text = \"%s\"\n",
	       gr->Axes[xlValue].AxisTitle.Text);
    }
  if (!gr->Axes[xlValue].MinimumScaleIsAuto)
    {
      fprintf (fp, "ActiveChart.MinimumScale(xlValue   ) = %f\n", gr->Axes[xlValue].MinimumScale);
    }
  if (!gr->Axes[xlValue].MaximumScaleIsAuto)
    {
      fprintf (fp, "ActiveChart.MaximumScale(xlValue   ) = %f\n", gr->Axes[xlValue].MaximumScale);
    }
  if (!gr->Axes[xlValue].MajorUnitIsAuto)
    {
      fprintf (fp, "ActiveChart.MajorUnit(xlValue   ) = %f\n", gr->Axes[xlValue].MajorUnit);
    }

  if (!gr->Axes[xlCategory].MinimumScaleIsAuto)
    {
      fprintf (fp, "ActiveChart.MinimumScale(xlCategory   ) = %f\n", gr->Axes[xlCategory].MinimumScale);
    }
  if (!gr->Axes[xlCategory].MaximumScaleIsAuto)
    {
      fprintf (fp, "ActiveChart.MaximumScale(xlCategory   ) = %f\n", gr->Axes[xlCategory].MaximumScale);
    }
  if (!gr->Axes[xlCategory].MajorUnitIsAuto)
    {
      fprintf (fp, "ActiveChart.MajorUnit(xlCategory   ) = %f\n", gr->Axes[xlCategory].MajorUnit);
    }

  return 0;
}

Graph *
ActivateGraph (chart)
     Graph *chart;
{
  ActiveGraph = chart;

  return ActiveGraph;
};

int
chart_settype (chart, type)
     Graph *chart;
     int type;
{

  if (chart != NULL);
  chart->type = type;
  return 0;
}

int
chart_setseriename (chart, serie, name)
     Graph *chart;
     int serie;
     char *name;
{
  if (chart != NULL);
  sprintf ((chart->range[serie]).name, "%s", name);
  return 0;
}

int
chart_setserievalues (gr, serie, valuerange)
     Graph *gr;
     int serie;
     char *valuerange;
{

  char *i1j1;
  char *i2j2;
  char *separ;
  if (serie < 0 || serie >= 10)
    return 1;
  if (gr == NULL)
    return 2;

  if (serie > gr->numrange)
    gr->numrange = serie;

  if (valuerange[0] == '=')
    valuerange++;

  separ = strchr (valuerange, ':');
  i1j1 = valuerange;
  i2j2 = separ + 1;
  separ[0] = '\0';

  alphatonum (i1j1, &(gr->range[serie]).i1, &(gr->range[serie]).j1);
  alphatonum (i2j2, &(gr->range[serie]).i2, &(gr->range[serie]).j2);

  if (serie + 1 > gr->numrange)
    gr->numrange = serie + 1;

  return 0;
}

int
chart_setgrid (gr, axe, on)
     Graph *gr;
     int axe;
     int on;
{
  if (gr == NULL)
    return 1;
  if (axe == 1)
    gr->Axes[xlCategory].HasMajorGridLines = on;
  if (axe == 2)
    gr->Axes[xlValue].HasMajorGridLines = on;
  return 0;
}

int
chart_setaxistitle (gr, axe, buf)
     Graph *gr;
     int axe;
     char *buf;
{
  if (gr == NULL)
    return 1;
  if (axe == 1)
    title_settext (&gr->Axes[xlCategory].AxisTitle, buf);
  if (axe == 2)
    title_settext (&gr->Axes[xlValue].AxisTitle, buf);
  return 0;
}

Axis *
chart_gaxis (Graph * gr, int axe)
{
  if (gr == NULL)
    return NULL;
  if (axe == 1)
    return &(gr->Axes[xlCategory]);
  if (axe == 2)
    return &(gr->Axes[xlValue]);
  return &(gr->Axes[xlValue]);
}
