/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "entrypop.h"
#include "param.h"
#include "list.xpm"
#include "memory.h"
#include "widget_makerX11.h"

static Entry_Pop **record;
static int nrecord = 0;

void
cb_entrypopEnter (w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     int *num_params;
{
  int i = 0;
  char *val;
  Widget item;
  TextFieldSetEditable (w, False);

  while (i < nrecord)
    {
      if (record[i]->entry == w)
	{
	  XtVaGetValues (w, XtNstring, &val, NULL);
	  item = entrypop_add_item (record[i], val);
	  if (record[i]->callback != NULL)
	    (*(record[i]->callback)) (item, XtName (item), NULL);
	  return;
	}
      i++;
    }
  return;
};

void
cb_entrypopEnterWin (Widget w, XtPointer pointer, XEvent * event, Boolean * ctd)
{

  XtSetKeyboardFocus ((Widget) gettop (), w);
  TextFieldSetEditable (w, True);
}

void
cb_entrypopLeaveWin (Widget w, XtPointer pointer, XEvent * event, Boolean * ctd)
{

}

void
cb_entrypopEdit (w, event, params, num_params)
     Widget w;
     XEvent *event;
     String *params;
     int *num_params;
{
};

int
entrypop_callback (pop, cbfunc)
     Entry_Pop *pop;
     void (*cbfunc) ();
{
  pop->callback = cbfunc;
  return 0;
}

void
changeentry (widget, pointer, junk)
     Widget widget;
     XtPointer pointer;
     XtPointer junk;
{
  int i = 0;
  Entry_Pop *ep_obj = (Entry_Pop *) pointer;
  while (widget != ep_obj->items[i] && i < ep_obj->nitem)
    i++;

  if (i == ep_obj->nitem)
    return;

  SetEntryString (ep_obj->entry, XtName (ep_obj->items[i]));
  TextFieldSetInsertionPosition (widget, 0);

  if (ep_obj->callback != NULL)
    (*(ep_obj->callback)) (ep_obj->items[i], XtName (ep_obj->items[i]), NULL);

}

int
entrypop_set_val (ep_obj, name)
     Entry_Pop *ep_obj;
     char *name;
{
  SetEntryString (ep_obj->entry, name);
  TextFieldSetInsertionPosition (ep_obj->entry, 0);
  return 0;
};

int
entrypop_init_val (ep_obj, name)
     Entry_Pop *ep_obj;
     char *name;
{
  SetEntryString (ep_obj->entry, name);
  return 0;
};

Widget
entrypop_add_item (ep_obj, name)
     Entry_Pop *ep_obj;
     char *name;
{
  Widget item;
  int pos = -1;
  int i;

  for (i = 0; i < ep_obj->nitem; i++)
    {
      int cmp = strcmp (XtName (ep_obj->items[i]), name);
      if (cmp == 0)
	return ep_obj->items[i];
      if (cmp > 0 && pos < 0)
	pos = i;
    }

  item = XtCreateManagedWidget (name,
				smeBSBObjectClass, ep_obj->popuplist,
				NULL, (Cardinal) 0);

  XtAddCallback (item, XtNcallback, changeentry, (XtPointer) ep_obj);

  ep_obj->nitem++;
  ep_obj->items = (Widget *) realloc (ep_obj->items, sizeof (Widget) * ep_obj->nitem);

  if (pos > 0)
    {
      for (i = ep_obj->nitem - 1; i > pos; i--)
	ep_obj->items[i] = ep_obj->items[i - 1];
      ep_obj->items[pos] = item;
    }
  else
    ep_obj->items[ep_obj->nitem - 1] = item;

  return item;
};

Entry_Pop *
newentrypop (parent, name)
     Widget parent;
     char *name;
{
  Entry_Pop *ep_obj;
  XtTranslations trans_table;
  Arg args[2];
  ep_obj = (Entry_Pop *) absmalloc (sizeof (Entry_Pop), "newentrypop:ep_obj ");

  record = (Entry_Pop **) absmalloc (sizeof (Entry_Pop *) * (nrecord + 1), "newentrypop:record ");
  record[nrecord] = ep_obj;;
  nrecord++;

  ep_obj->parent = parent;

  w_n = 0;
  w_dim (50, 17);
  w_bord (0);
  w_inbord (0);
  w_noresize ();
  ep_obj->baseform = XtCreateManagedWidget ("entrypopbase", formWidgetClass, ep_obj->parent, w_args, w_n);

  ep_obj->entry = CreateEntry (ep_obj->baseform, "entry", 0, 15, 35, 15, " ");
  w_n = 0;
  w_rel (NULL, 0, 0);
  w_bONw ();
  w_arg (XtNlength, 4);
  w_set (ep_obj->entry);

  TextFieldSetEditable (ep_obj->entry, False);

  ep_obj->flechbtn = (Widget) make_menu_button_pixmap (ep_obj->baseform, "list", list_xpm, NULL, NULL);
  w_n = 0;
  w_relv (NULL, 0);
  w_bONw ();
  w_relh (ep_obj->entry, 0);
  w_set (ep_obj->flechbtn);

  XtSetArg (args[0], XtNmenuName, name);
  XtSetValues (ep_obj->flechbtn, args, (Cardinal) 1);

  ep_obj->popuplist = XtCreatePopupShell (name, simpleMenuWidgetClass, ep_obj->flechbtn, NULL, 0);

  ep_obj->nitem = 0;
  ep_obj->items = NULL;
  ep_obj->callback = NULL;

  XtAppAddActions (XtWidgetToApplicationContext (ep_obj->entry)
		   ,entrypop_actionsTable, XtNumber (entrypop_actionsTable));

  trans_table = XtParseTranslationTable (entrypop_Translations);
  XtOverrideTranslations (ep_obj->entry, trans_table);

  XtAddEventHandler (ep_obj->entry, EnterWindowMask, True, cb_entrypopEnterWin, (XtPointer) ep_obj);
  XtAddEventHandler (ep_obj->entry, LeaveWindowMask, True, cb_entrypopLeaveWin, (XtPointer) ep_obj);

  return ep_obj;

}
