/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "libfct.h"
#include "y.tab.h"
#include "string.h"
#include "mathfct.h"
#include "textfct.h"
#include "statfct.h"
#include "informationfct.h"
#include "memory.h"

static Fct matrixfct2[] =
{
  {"CELLS", &vb_gcell, CELL, 2, NULL, NULL},
  {"WKSCELLS", &vb_worksheet_gcell, CELL, 2, NULL, NULL},
  {"Range", &vb_grange, STRING, 1, NULL, NULL},
  {"Worksheets", &vb_gworksheet, WORKSHEET, 1, NULL, NULL},
  {"Workbooks", &vb_gworkbook, WORKBOOK, 1, NULL, NULL},
  {"Columns", &vb_gcolumn, COLUMN, 1, NULL, NULL},
  {"Rows", &vb_grow, ROW, 1, NULL, NULL},
  {"SeriesCollection", &vb_gserie, SERIE, 1, NULL, NULL},
  {"Axes", &vb_gaxis, AXIS, 1, NULL, NULL},
  {"PRINT", &vb_print, 1, 1, NULL, NULL},
  {"REPAINT", &vb_repaint, 1, 1, NULL, NULL},
  {"SHELL", &vb_shell, INTEGER_CONSTANT, 1, NULL, NULL},
  {NULL, NULL, 0, 0, NULL, NULL}
};

obj
vb_print (narg, arg)
     int narg;
     obj *arg;
{
  obj o;
  int i;
  for (i = 0; i < narg; i++)
    niceprintobj (arg[i]);
  printf ("\n");
  return o;
};

obj
vb_repaint (narg, arg)
     int narg;
     obj *arg;
{
  obj o;
  xrepnt ();
  return o;
};

obj
vb_shell (narg, arg)
     int narg;
     obj *arg;
{
  obj o;
  if (narg == 1)
    {
      system (obj2string (arg[0]));
    }
  return o;
};

static int nfct = 0;
Fct *matrixfct = NULL;

int
addfunction (char *name, obj (*fct) (), int rettype, int narg, char *proto, char *comment)
{
  matrixfct = (Fct *) absrealloc (matrixfct, sizeof (Fct) * (nfct + 2), "addfunction:matrixfct");
  matrixfct[nfct].name = name;
  matrixfct[nfct].fct = fct;
  matrixfct[nfct].rettype = rettype;
  matrixfct[nfct].narg = narg;
  matrixfct[nfct].proto = proto;
  matrixfct[nfct].comment = comment;
  nfct++;
  matrixfct[nfct].name = NULL;
  matrixfct[nfct].fct = NULL;

  return 0;
}

int
removefunction (char *name)
{
  return 0;
}

static int
addmatrixfunction (Fct matrix[])
{
  int i = 0;
  while (matrix[i].name != NULL)
    {
      addfunction (matrix[i].name, matrix[i].fct,
      matrix[i].rettype, matrix[i].narg, matrix[i].proto, matrix[i].comment);
      i++;
    }
  return 0;
}

int
initbuiltinfunction ()
{
  addmatrixfunction (matrixfct2);
  addmatrixfunction (mathmatrixfct);
  addmatrixfunction (textmatrixfct);
  addmatrixfunction (statmatrixfct);
  addmatrixfunction (informationmatrixfct);
}

static double *matrixdouble = NULL;
static int dimmatrixdouble = 0;

double *
getmatrixdouble ()
{
  return matrixdouble;
};
int
freematrixdouble ()
{
  if (matrixdouble != NULL)
    absfree (matrixdouble, "freematrixdouble");
  matrixdouble = NULL;
  return 0;
}

int
arg2matrixdouble (int narg, obj * arg)
{
  int dim = 0;
  int pos = 0;
  int i, r, c;

  for (i = 0; i < narg; i++)
    {
      if (arg[i].type == RANGE)
	{
	  tmpRange *ran = (tmpRange *) arg[i].rec.s;
	  dim += (ran->r2 - ran->r1 + 1) * (ran->c2 - ran->c1 + 1);
	  if (dim != 0);
	  matrixdouble = (double *) absrealloc (matrixdouble, dim * sizeof (double), "arg2matrixdouble");
	  if (ran->wks == NULL)
	    {
	      for (r = ran->r1; r <= ran->r2; r++)
		for (c = ran->c1; c <= ran->c2; c++)
		  {
		    matrixdouble[pos] = get_value (r, c);
		    pos++;
		  }
	    }
	  else
	    {
	      for (r = ran->r1; r <= ran->r2; r++)
		for (c = ran->c1; c <= ran->c2; c++)
		  {
		    matrixdouble[pos] = get_value_wks (ran->wks, r, c);
		    pos++;
		  }
	    }
	}
      else
	{
	  dim++;
	  matrixdouble = (double *) absrealloc (matrixdouble, dim * sizeof (double), "arg2matrixdouble");
	  matrixdouble[pos] = obj2double (arg[i]);
	  pos++;
	}
    }
  dimmatrixdouble = dim;
  return dim;
}
