/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#ifndef LIBFCT_H
#define LIBFCT_H
#include "math.h"
#include "object.h"
#include "cell_vb.h"
#include "font_vb.h"
#include "tbd_vb.h"
#include "worksheet_vb.h"
#include "workbook_vb.h"
#include "application_vb.h"
#include "interior_vb.h"
#include "row_vb.h"
#include "column_vb.h"
#include "border_vb.h"
#include "chart_vb.h"
#include "axis_vb.h"
#include "title_vb.h"
#include "button_vb.h"
#include "y.tab.h"


/*type definition = position in the matrixclass!*/
#include "typedef.h"

/*****************/

typedef struct
 { char* name;
   obj (*fct) ();
   int rettype;
   int narg;
   char * proto;
   char * comment;
 } Fct;

extern int addfunction (char* name, obj (*fct)(),int rettype,int narg,char* proto,char* comment);
extern int removefunction (char* name);
extern int initbuiltinfunction();

extern double* getmatrixdouble();
extern int freematrixdouble() ;
extern int arg2matrixdouble (int narg, obj* arg);


extern Fct * matrixfct;



typedef struct
{ char* name;
  int type;
  obj (*setfct) ();
  obj (*getfct) ();
} Memberdata;

typedef struct
{ char *name;
  int type;
  Memberdata* data;
  Fct* fct;
} Class;

/*======================== application =======================*/
static Memberdata Applicationdata [] =
{ {"ActiveWorkbook",  WORKBOOK, &vb_application_setactiveworkbook,  &vb_application_gactiveworkbook  },
  {"ActiveCell",  CELL, &vb_application_setactivecell,  &vb_application_gactivecell  },
  {"ActiveWorksheet",  WORKSHEET, &vb_application_setactiveworksheet,  &vb_application_gactiveworksheet  },
  {"Thisworkbook",  WORKBOOK, &vb_application_setthisworkbook,  &vb_application_gthisworkbook  },
  {"ScreenUpdating",  INTEGER,&vb_application_setscreenupdating, &vb_application_gscreenupdating },
  {NULL,     0   ,NULL,NULL}
};

static Fct Applicationfct [] =
{ {"Quit",&vb_application_quit,INTEGER,0},
  {"Calculate",&vb_application_calculate,INTEGER,0},
  {NULL,NULL,0,0}
};

/*======================== workbooks =======================*/
static Memberdata Workbooksdata [] =
{ {"Count",  INTEGER,&vb_workbooks_setcount,  &vb_workbooks_gcount  },
  {NULL,     0   ,NULL,NULL}
};

static Fct Workbooksfct [] =
{ {"Add",&vb_workbooks_add,WORKBOOK,0},
  {"Open",&vb_workbooks_o,WORKBOOK,5},
  {"Close",&vb_workbooks_close,INTEGER,0},
  {NULL,NULL,0,0}
};

/*======================== workbook =======================*/
static Memberdata Workbookdata [] =
{ {"ActiveSheet",  WORKSHEET,&vb_workbook_setactivesheet,  &vb_workbook_gactivesheet  },
  {NULL,     0   ,NULL,NULL}
};

static Fct Workbookfct [] =
{ {"Activate",&vb_workbook_activate,WORKBOOK,0},
  {"Calculate",&vb_workbook_calculate,INTEGER,0},
  {"Close",&vb_workbook_close,INTEGER,0},
  {"PrintOut",&vb_workbook_printout,INTEGER,6},
  {"Save",&vb_workbook_s,INTEGER,0},
  {"SaveAs",&vb_workbook_sas,INTEGER,0},
  {NULL,NULL,0,0}
};

/*======================== worksheets =======================*/
static Memberdata Worksheetsdata [] =
{ {"Count",  INTEGER,&vb_worksheets_setcount,  &vb_worksheets_gcount  },
  {NULL,     0   ,NULL,NULL}
};

static Fct Worksheetsfct [] =
{ {"Add",&vb_worksheets_add,WORKSHEET,0},
  {NULL,NULL,0,0}
};

/*======================== worksheet =======================*/
static Memberdata Worksheetdata [] =
{ {"name",  STRING,&vb_worksheet_setname,  &vb_worksheet_gname  },
  {"ChartObjects",  OBJECT,NULL,NULL  },
  {"Shapes",  OBJECT,NULL,NULL  },
  {"Buttons",  OBJECT,NULL,NULL  },
  {NULL,     0   ,NULL,NULL}
};

static Fct Worksheetfct [] =
{ {"Activate",&vb_worksheet_activate,WORKSHEET,0},
  {"Calculate",&vb_worksheet_calculate,INTEGER,0},
  {"Copy",&vb_worksheet_copy,INTEGER,2},
  {"Delete",&vb_worksheet_delete,INTEGER,0},
  {"Move",&vb_worksheet_move,INTEGER,2},
  {"Paste",&vb_worksheet_paste,INTEGER,2},
  {"PrintOut",&vb_worksheet_printout,INTEGER,0},
  {"Cells",&vb_worksheet_gcell,CELL,2},
  {"ChartObjects",&vb_worksheet_gchart,CHARTOBJECT,2},
  {NULL,NULL,0,0}
};

/*======================== cell =======================*/

static Memberdata Celldata [] =
{ {"castobj",VARIANT,&vb_cell_setobj,  &vb_cell_gobj  },
  {"value",  VARIANT,&vb_cell_setvalue,  &vb_cell_gvalue  },
  {"formula",STRING,&vb_cell_setformula,&vb_cell_gformula},
  {"font",   OBJECT,NULL               ,NULL              },
  {"numberformat",   STRING,&vb_cell_setnumform,&vb_cell_gnumform   },
  {"interior",   OBJECT,&vb_cell_setinterior ,&vb_cell_ginterior     },
  {"HorizontalAlignment", INTEGER,&vb_cell_sethalign ,&vb_cell_ghalign  },
  {"Borders", BORDER ,&vb_cell_setbord ,&vb_cell_gbord  },
  {NULL,     0   ,NULL,NULL}
};

static Fct Cellfct [] =
{ {"=",&vb_cell_assign,CELL,2},
  {"select",&vb_cell_select,0,1},
  {"PrintOut",&vb_cell_print,0,1},
  {"Borders",&vb_cell_gborder,BORDER,1},
  {NULL,NULL,0,0}
};

/*========================= font ======================*/
static Memberdata Fontdata [] =
{ {"bold",       BOOLEAN,&vb_font_setbold,    &vb_font_gbold},
  {"italic",     BOOLEAN,&vb_font_setitalic,  &vb_font_gitalic},
  {"name",       STRING ,&vb_font_setname,  &vb_font_gname},
  {"size",       INTEGER,&vb_font_setsize,  &vb_font_gsize},
  {"ColorIndex", INTEGER,&vb_font_setcolorindex,  &vb_font_gcolorindex},
  {NULL,         0   ,NULL,NULL}
};

static Fct Fontfct [] =
{ {"=",&vb_font_assign,CELL,2},
  {"print",&vb_font_print,0,1},
  {NULL,NULL,0,0}
};

/*========================= interior======================*/
static Memberdata Interiordata [] =
{ {"ColorIndex", INTEGER,&vb_interior_setcolorindex,  &vb_interior_gcolorindex},
  {NULL,         0   ,NULL,NULL}
};

static Fct Interiorfct [] =
{  {NULL,NULL,0,0}
};


/*========================= column======================*/
static Memberdata Columndata [] =
{ {"ColumnWidth", DOUBLE,&vb_column_setwidth,  &vb_column_gwidth},
  {NULL,         0   ,NULL,NULL}
};

static Fct Columnfct [] =
{  {NULL,NULL,0,0}
};

/*========================= row======================*/
static Memberdata Rowdata [] =
{ {"RowHeight", DOUBLE,&vb_row_setheight,  &vb_row_gheight},
  {NULL,         0   ,NULL,NULL}
};

static Fct Rowfct [] =
{  {NULL,NULL,0,0}
};

/*========================= border======================*/
static Memberdata Borderdata [] =
{ {"LineStyle", INTEGER,&vb_border_setlinestyle,  &vb_border_glinestyle},
  {NULL,         0   ,NULL,NULL}
};

static Fct Borderfct [] =
{  {NULL,NULL,0,0}
};

/*========================= shapes======================*/
static Memberdata Shapesdata [] =
{  {NULL,         0   ,NULL,NULL}
};

static Fct Shapesfct [] =
{  {"AddShape",&vb_shapes_add,5,INTEGER_CONSTANT},
   {"AddLine", &vb_shapes_add,4,INTEGER_CONSTANT},
   {NULL,        NULL   ,0 ,0}
};

/*========================= buttons ======================*/
static Memberdata Buttonsdata [] =
{  {NULL,         0   ,NULL,NULL}
};

static Fct Buttonsfct [] =
{  {"Add",&vb_buttons_add,4,BUTTON},
   {NULL,        NULL   ,0 ,0}
};
/*========================= button ======================*/
static Memberdata Buttondata [] =
{ {"OnAction",  STRING_CONSTANT, &vb_button_setonaction, &vb_button_gonaction  },
  {"Text",      STRING_CONSTANT, &vb_button_settext, &vb_button_gtext  },
  {NULL,         0   ,NULL,NULL}
};

static Fct Buttonfct [] =
{ 
   {NULL,        NULL   ,0 ,0}
};



/*========================= chartobjects======================*/
static Memberdata Chartobjectsdata [] =
{  {NULL,         0   ,NULL,NULL}
};

static Fct Chartobjectsfct [] =
{  {"Add",&vb_chartobjects_add,4,CHARTOBJECT},
   {"Delete",&vb_chartobjects_delete,0,INTEGER_CONSTANT},
   {NULL,        NULL   ,0 ,0}
};



/*========================= chartobject======================*/
static Memberdata Chartobjectdata [] =
{ {"Chart",  CHART, &vb_chartobject_setchart, &vb_chartobject_gchart  },
  {"Border", OBJECT, NULL,  NULL   },
  {"Interior", OBJECT, NULL,  NULL   },
  {NULL,         0   ,NULL,NULL}
};

static Fct Chartobjectfct [] =
{  {"Activate",&vb_chartobject_activate,1,CHART},
    {NULL,        NULL   ,0 ,0}
};

/*========================= chart======================*/
static Memberdata Chartdata [] =
{ {"ChartTitle", TITLE,  &vb_chart_settitle,      &vb_chart_gtitle   },
  {"HasTitle",   BOOLEAN,&vb_chart_sethastitle,   &vb_chart_ghastitle},
  {"ChartType",  INTEGER,&vb_chart_setcharttype,  &vb_chart_gcharttype},
  {"SeriesCollection",OBJECT,NULL,NULL},
  {NULL,         0   ,NULL,NULL}
};

static Fct Chartfct [] =
{  {"Axes",            &vb_chart_gaxis,1,AXIS},
   {"SeriesCollection",&vb_chart_gserie,1,SERIE},
    {NULL,        NULL   ,0 ,0}
};

/*========================= serie======================*/
static Memberdata Seriedata [] =
{ {"XValues", INTEGER ,  &vb_serie_setxvalues,      &vb_serie_gxvalues   },
  {"Name",    INTEGER,  &vb_serie_setname,         &vb_serie_gname      },
  {"Values",  INTEGER,   &vb_serie_setvalues ,      &vb_serie_gvalues    },
  {NULL,         0   ,NULL,NULL}
};

static Fct Seriefct [] =
{    {NULL,        NULL   ,0 ,0}
};

/*========================= seriescollection======================*/
static Memberdata Seriescollectiondata [] =
{
  {NULL,         0   ,NULL,NULL}
};

static Fct Seriescollectionfct [] =
{  {"NewSerie",&vb_seriescollection_nserie,0,0},
    {NULL,        NULL   ,0 ,0}
};



/*========================= axis======================*/
static Memberdata Axisdata [] =
{ {"AxisTitle", TITLE, &vb_axis_settitle,      &vb_axis_gtitle  },
  {"HasTitle",   BOOLEAN,&vb_axis_sethastitle,  &vb_axis_ghastitle},
  {"MinimumScale",  DOUBLE,&vb_axis_setminimumscale,  &vb_axis_gminimumscale},
  {"MaximumScale",  DOUBLE,&vb_axis_setmaximumscale,  &vb_axis_gmaximumscale},
  {"Minorunit",  DOUBLE,&vb_axis_setminorunit,  &vb_axis_gminorunit},
  {"Majorunit",  DOUBLE,&vb_axis_setmajorunit,  &vb_axis_gmajorunit},
  {"MajorGridLines",  BOOLEAN,&vb_axis_setmajorgridlines,  &vb_axis_gmajorgridlines},
  {"MinorGridLines",  BOOLEAN,&vb_axis_setminorgridlines,  &vb_axis_gminorgridlines},
  {NULL,         0   ,NULL,NULL}
};

static Fct Axisfct [] =
{  {NULL,NULL,0,0}
};

/*========================= title======================*/
static Memberdata Titledata [] =
{ {"Text", STRING_CONSTANT, &vb_title_settext,   &vb_title_gtext   },
  {NULL,         0   ,NULL,NULL}
};

static Fct Titlefct [] =
{  {NULL,NULL,0,0}
};



/*==================== ARRAY CLASS =======================*/
static Class matrixclass [] =
{ {"WORKBOOK",  WORKBOOK,   Workbookdata, Workbookfct     },
  {"WORKSHEET", WORKSHEET,  Worksheetdata,     Worksheetfct},
  {"Cell",      CELL,       Celldata, Cellfct},
  {"Font",      FONT,       Fontdata, Fontfct},
  {"Column",    COLUMN,     Columndata,   Columnfct},
  {"Border",    BORDER,     Borderdata,   Borderfct},
  {"Row",       ROW   ,     Rowdata,     Rowfct},
  {"Serie",     SERIE   , Seriedata,  Seriefct},
  {"Axis",      AXIS  ,     Axisdata,     Axisfct},
  {"Interior",  INTERIOR  , Interiordata,  Interiorfct},
  {"ChartObjects", CHARTOBJECTS  ,    Chartobjectsdata,  Chartobjectsfct},
  {"ChartObject", CHARTOBJECT  ,    Chartobjectdata,  Chartobjectfct},
  {"Chart",     CHART  ,    Chartdata,  Chartfct},
  {"ChartTitle", CHARTTITLE   , Titledata,  Titlefct},
  {"AxisTitle", AXISTITLE   , Titledata,  Titlefct},
  {"Title", TITLE   , Titledata,  Titlefct},
  {"SeriesCollection", SERIESCOLLECTION , Seriescollectiondata,  Seriescollectionfct},
  {"Range", RANGE , Celldata, Cellfct},
  {"Application", APPLICATION , Applicationdata, Applicationfct },
  {"shapes", SHAPES , Shapesdata,  Shapesfct},
  {"WORKBOOKS",  WORKBOOKS,   Workbooksdata, Workbooksfct     },
  {"WORKSHEETS", WORKSHEETS,  Worksheetsdata,     Worksheetsfct},
  {"BUTTONS", BUTTONS,  Buttonsdata,     Buttonsfct},
  {"BUTTON", BUTTON,  Buttondata,     Buttonfct},
  {NULL,        0,          NULL,     NULL}
};

 
  

#endif /* LIBFCT_H*/
