/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "param.h"
#include <stdio.h>
#include "mainwin.h"
#include "worksheet.h"
#include "memory.h"

extern int lindrag (double l);
extern int coldrag (double l);

static String fallback_resources[] =
{
  "*window.width:         800",
  "*window.height:        600",
  "abs*background:         #C000C000C000",
  "abs*foreground:         black",
  "*keyboardFocusPolicy:           explicit",
  NULL
};

static XtActionsRec actionsTable[] =
{
  {"DocBtn1Up", cbdb1up},
  {"PopDown", cb_popdown},
  {"DocMotion", cbdmv},
  {"DocBtn1Motion", cb_DocBtn1Motion},
  {"DocBtn1Down", cbdb1down},
  {"DocBtn2Up", cbdb2up},
  {"DocBtn2Motion", cbdbt2mv},
  {"DocBtn2Down", cbdn2down},
  {"TextReturn", cb_TextReturn},
  {"TextUp", cbtup},
  {"TextDown", cbtdown},
  {"TextLeft", cbtleft},
  {"TextRight", cbtright},
  {"DocBtn3Up", cb_DocBtn3Up},
  {"Delete", cbdel},
  {"DocDelete", cb_DocDelete},
  {"BackSpace", cbbak},
  {"PageUp", cbpgup},
  {"PageDown", cbpgdwn},
  {"PageLeft", cbpgleft},
  {"PageRight", cbpgrig},
  {"PageHome", cb_PageHome},
  {"Break", cb_Break},
};
static char defaultTranslations[] = "\
<Btn1Up>:     DocBtn1Up()\n\
<Btn1Motion>: DocBtn1Motion()\n\
<Btn2Up>:     DocBtn2Up()\n\
<Btn2Motion>: DocBtn2Motion()\n\
<Motion>:    DocMotion()\n\
<Btn1Down>:  PopDown() DocBtn1Down()\n\
<Btn2Down>:  PopDown() DocBtn2Down()\n\
<Btn3Down>:  PopDown()\n\
<Key>Delete: DocDelete() \n\
<Key>Page_Up: PageUp() \n\
<Key>Page_Down: PageDown() \n\
Shift<Key>Tab: PageLeft() \n\
<Key>Tab: PageRight() \n\
<Key>Home: PageHome() \n\
Ctrl<Key>B: Break() \n\
";

static char textTranslations[] = "\
<Key>Return:   TextReturn(%d)\n\
<Key>KP_Enter:   TextReturn(%d)\n\
<Key>osfUp:    TextUp() \n\
<Key>osfDown:   TextDown() \n\
<Key>osfLeft:   TextLeft() \n\
<Key>osfRight:   TextRight() \n\
<Key>Up:    TextUp() \n\
<Key>Down:  TextDown() \n\
<Key>Left:  TextLeft() \n\
<Key>Right:  TextRight() \n\
<Key>BackSpace: BackSpace() \n\
<Key>Delete: Delete() \n\
";

static char alltextTranslations[] = "\
<Key>BackSpace: BackSpace() \n\
<Key>Delete: Delete() \n\
";

static char popupTranslations[] = "\
<Motion>:       unhighlight() highlight()\n\
<Btn1Up>:           PopDown() notify() unhighlight()\n\
";

Mainwin *ActiveMainwin = NULL;

void
scrollv (v)
     int v;
{
  double V = v;
  lindrag (V);
  return;
}

void
scrollh (v)
     int v;
{
  double V = v;
  coldrag (V);
  return;
}

void
setscrollv (double v)
{
  setscrollval (ActiveMainwin->scrollv, v);
}

void
setscrollh (double v)
{
  setscrollval (ActiveMainwin->scrollh, v);
}

void
setscrollvlimit (vmin, vmax)
     int vmin, vmax;
{
  setscrolllimit (ActiveMainwin->scrollv, vmin, vmax);
}

void
setscrollhlimit (vmin, vmax)
     int vmin, vmax;
{
  setscrolllimit (ActiveMainwin->scrollh, vmin, vmax);
}

void
mainwin_resize (widget, pointer, event, ctd)
     Widget widget;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int n;
  Dimension h, w;
  Widget wi;
  Arg args[2];
  wi = (Widget) pointer;

  n = 0;
  XtSetArg (args[n], XtNheight, &h);
  n++;
  XtSetArg (args[n], XtNwidth, &w);
  n++;
  XtGetValues (wi, args, n);
  cb_resize (w, h);
};

Mainwin *
newmainwin (Widget top)
{
  XtTranslations trans_table;
  XtTranslations text_trans_table;
  XtTranslations popup_trans_table;
  Mainwin *mainwin = (Mainwin *) absmalloc (sizeof (Mainwin), "newmainwin:mainwin");
  ActiveMainwin = mainwin;

  mainwin->toplevel = top;

  w_n = 0;
  w_dim (800, 600);
  w_inbord (0);
  w_arg (XtNdefaultDistance, 1);
  mainwin->baseform = XtCreateManagedWidget
    ("main_window", formWidgetClass, mainwin->toplevel, w_args, w_n);

  w_n = 0;
  w_dim (800, 20);
  w_rel (NULL, 0, 0);
  w_bord (0);
  w_inbord (0);
  w_noresize ();
  mainwin->menubar = XtCreateManagedWidget
    ("menubar", boxWidgetClass, mainwin->baseform, w_args, w_n);
  mainwin->menubar_obj = (Menubar *) newmenubar (mainwin->menubar);

  XtAddEventHandler (mainwin->menubar, ExposureMask, FALSE, shadow, (XtPointer) mainwin->menubar);

  w_n = 0;
  w_dim (800, 20);
  w_relh (NULL, 0);
  w_relv (mainwin->menubar, 0);
  w_bord (0);
  w_inbord (0);
  w_noresize ();
  mainwin->buttonbar = XtCreateManagedWidget
    ("buttonbar", boxWidgetClass, mainwin->baseform, w_args, w_n);
  XtAddEventHandler (mainwin->buttonbar, ExposureMask, FALSE, shadow, (XtPointer) mainwin->buttonbar);

  w_n = 0;
  w_dim (800, 25);
  w_relh (NULL, 0);
  w_relv (mainwin->buttonbar, 0);
  w_bord (0);
  w_novresize ();
  w_hresize ();
  mainwin->commandline = XtCreateManagedWidget
    ("commandline", formWidgetClass, mainwin->baseform, w_args, w_n);

  mainwin->commandline_obj = newcommandline (mainwin->commandline);

  w_n = 0;
  w_relh (NULL, 0);
  w_relv (mainwin->commandline, 0);
  w_bord (0);
  w_arg (XtNleft, XtChainLeft);
  w_arg (XtNtop, XtChainTop);
  w_arg (XtNbottom, XtChainBottom);
  w_arg (XtNright, XtChainRight);
  w_arg (XtNdefaultDistance, 0);

  mainwin->wbform = (Widget) XtCreateManagedWidget ("window", formWidgetClass, mainwin->baseform,
						    w_args, w_n);
  XtAddEventHandler (mainwin->wbform, ExposureMask, FALSE, shadow, (XtPointer) mainwin->wbform);
  XtAddEventHandler (mainwin->wbform, StructureNotifyMask, False, mainwin_resize, (XtPointer) mainwin->wbform);

  w_n = 0;
  w_dim (30, 20);
  w_rel (NULL, 0, 0);
  w_bord (0);
  w_noresize ();
  mainwin->corner = (Widget) XtCreateManagedWidget ("window", simpleWidgetClass, mainwin->wbform,
						    w_args, w_n);

  w_n = 0;
  w_dim (30, 510);
  w_relh (NULL, 0);
  w_relv (mainwin->corner, 0);
  w_bord (0);
  w_nohresize ();
  w_vresize ();
  w_arg (XtNtop, XtChainTop);
  mainwin->linind = (Widget) XtCreateManagedWidget ("window", panedWidgetClass, mainwin->wbform,
						    w_args, w_n);

  w_n = 0;
  w_dim (784, 20);
  w_relv (NULL, 0);
  w_relh (mainwin->corner, 0);
  w_bord (0);
  w_novresize ();
  w_hresize ();
  w_arg (XtNleft, XtChainLeft);
  mainwin->colind = (Widget) XtCreateManagedWidget ("window", panedWidgetClass, mainwin->wbform,
						    w_args, w_n);

  w_n = 0;
  w_dim (784, 510);
  w_relv (mainwin->colind, 0);
  w_relh (mainwin->linind, 0);
  w_bord (0);
  w_hresize ();
  w_vresize ();
  mainwin->draw = (Widget) XtCreateManagedWidget ("window", panedWidgetClass, mainwin->wbform,
						  w_args, w_n);

  w_n = 0;
  w_relv (mainwin->commandline, 0);
  w_relh (mainwin->wbform, 1);
  w_bord (0);
  w_arg (XtNtop, XtChainTop);
  w_arg (XtNleft, XtChainRight);
  w_arg (XtNright, XtChainRight);
  w_arg (XtNbottom, XtChainBottom);
  w_arg (XtNdefaultDistance, 1);
  mainwin->scrollv = CreateMyscroll (mainwin->baseform, 528, 14, 0, 400, scrollv);

  w_n = 0;
  w_dim (80, 20);
  w_relh (NULL, 0);
  w_relv (mainwin->wbform, 0);
  w_bord (0);
  w_inbord (0);
  w_arg (XtNdefaultDistance, 0);
  w_arg (XtNtop, XtChainBottom);
  w_arg (XtNbottom, XtChainBottom);
  w_arg (XtNleft, XtChainLeft);
  w_arg (XtNright, XtChainLeft);
  mainwin->sheetbtn = XtCreateManagedWidget ("window", formWidgetClass, mainwin->baseform,
					     w_args, w_n);

  w_n = 0;
  w_dim (320, 20);
  w_relh (NULL, 80);
  w_relv (mainwin->wbform, 0);
  w_bord (0);
  w_inbord (0);
  w_arg (XtNtop, XtChainBottom);
  w_arg (XtNbottom, XtChainBottom);
  w_arg (XtNleft, XtChainLeft);
  w_arg (XtNright, XtChainLeft);
  mainwin->sheetselform = XtCreateManagedWidget ("window", formWidgetClass, mainwin->baseform,
						 w_args, w_n);
  build_thumb (mainwin->sheetselform);

  w_n = 0;
  w_relv (mainwin->wbform, 1);
  w_relh (NULL, 453);
  w_bord (0);
  w_arg (XtNtop, XtChainBottom);
  w_arg (XtNbottom, XtChainBottom);
  w_arg (XtNleft, XtChainLeft);
  w_arg (XtNright, XtChainRight);
  w_arg (XtNdefaultDistance, 1);
  mainwin->scrollh = CreateMyscroll (mainwin->baseform, 14, 360, 0, NBCOL, scrollh);

  mainwin->buttonbar_obj = newbuttonbar (mainwin->buttonbar);

  mainwin->popup = (Widget) make_alone_popup (mainwin->wbform, "pop1");

  XtAddEventHandler (mainwin->draw, ExposureMask, FALSE, cb_Repaint, NULL);
  XtAddEventHandler (mainwin->draw, StructureNotifyMask, FALSE, cb_Repaint, NULL);

  XtAppAddActions (XtWidgetToApplicationContext (mainwin->toplevel), actionsTable, XtNumber (actionsTable));

  trans_table = XtParseTranslationTable (defaultTranslations);
  text_trans_table = XtParseTranslationTable (textTranslations);
  popup_trans_table = XtParseTranslationTable (popupTranslations);

  XtOverrideTranslations (mainwin->commandline_obj->text, text_trans_table);

  XtOverrideTranslations (mainwin->popup, popup_trans_table);

  XtOverrideTranslations (mainwin->draw, trans_table);
  XtOverrideTranslations (mainwin->linind, trans_table);
  XtOverrideTranslations (mainwin->colind, trans_table);
  XtOverrideTranslations (mainwin->corner, trans_table);

  XtAddEventHandler (mainwin->draw, ButtonReleaseMask, True, cb_DocBtn3Up,
		     (XtPointer) mainwin->popup);

  XtAddEventHandler (mainwin->commandline_obj->text, KeyReleaseMask, True, cb_TextChange, mainwin->commandline_obj->text);

  XtRealizeWidget (mainwin->toplevel);

  return mainwin;
}

int
mainwin_popup_abv ()
{
  XtManageChild (ActiveMainwin->abv_editor);
}

int
mainwin_popdown_abv ()
{
  XtUnmanageChild (ActiveMainwin->abv_editor);
}
