/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "menubar.h"
#include "separator.h"
#include "memory.h"
#include "file_selector.h"
#include "separator.h"
#include "main.h"
#include "application.h"
#include "macrodialog.h"

void
nocb ()
{
};

void
X11m_ofile ()
{
  MakePrompt (gettop (), "open file:", openfl, "defaut.abs");
}

void
X11m_sfile ()
{
  savefl ();
}

void
X11m_sasfile ()
{
  MakePrompt (gettop (), "save file:", saveasfile, "defaut.abs");

}

#ifdef ABS_SAMCEF
void
X11m_ofac ()
{
  MakePrompt (gettop (), "open fac:", inifac, "defaut.abs");
}
#endif

void
X11m_insertfl ()
{
  X11m_separator (INSERTFILE);

}

void
X11m_exportfile ()
{
  X11m_separator (EXPORTFILE);

}

void
X11m_macro ()
{
  MacroMakePrompt (gettop (), "Macro", playmacro, "defaut.abs");
}

void
X11m_formatborder ()
{

}

void
m_print (char *filename)
{
  printfl (filename);
}

void
m_close ()
{
  close_doc ();
};

Widget
make_bar_button (menu_bar, name)
     char *name;
     Widget menu_bar;
{
  Arg args[20];
  register int n;
  n = 0;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  return XtCreateManagedWidget
    (name, menuButtonWidgetClass, menu_bar, args, n);
}

Widget
make_popup (bar_button, name, button_name, menuentry, num)
     char *name;
     char *button_name;
     Widget bar_button;
     struct menuEntry *menuentry;
     int num;
{
  Arg args[20];
  register int n;
  Widget popupmenu, entry;
  int i;
  n = 0;

  XtSetArg (args[0], XtNmenuName, name);
  XtSetValues (bar_button, args, (Cardinal) 1);
  popupmenu = XtCreatePopupShell
    (name, simpleMenuWidgetClass, bar_button, args, n);
  for (i = 0; i < num; i++)
    {
      entry = XtCreateManagedWidget (menuentry[i].name,
				     smeBSBObjectClass, popupmenu,
				     NULL, (Cardinal) 0);
      XtAddCallback (entry, XtNcallback, menuentry[i].function, NULL);
    }

  return popupmenu;

}

Widget
add_entry (popupmenu, name, func)
     Widget popupmenu;
     char *name;
     void (*func) ();
{
  Widget entry = XtCreateManagedWidget (name,
					smeBSBObjectClass, popupmenu,
					NULL, (Cardinal) 0);
  XtAddCallback (entry, XtNcallback, func, NULL);
  return entry;
};

static level;
static Widget popupmenu[10];
static Widget windowname[20];
static int nwin = 0;

int
add_window_name (name)
     char *name;
{
  int i = 0;
  while (i < 10 && strncmp (rootmenu[i].name, "Windows", 7))
    i++;

  if (nwin == 20)
    return -1;

  windowname[nwin] = add_entry (popupmenu[i], name, m_changewin);
  nwin++;
  return 0;
}

int
remove_window_name (name)
     char *name;
{
  int j;
  int i = 0;

  while (i < nwin && strcmp (XtName (windowname[i]), name))
    i++;

  if (i == nwin)
    return 0;

  XtUnmanageChild (windowname[i]);
  XtDestroyWidget (windowname[i]);

  nwin--;

  for (j = i; j < nwin; j++)
    windowname[j] = windowname[j + 1];

  return 0;
}

int
recursive_build (mb_obj, parent, menu)
     Menubar *mb_obj;
     Widget parent;
     Menu_ele *menu;
{
  int i;
  Arg args[20];
  register int n = 0;
  Widget entry, button;
  XtTranslations menu_trans_table;
  XtTranslations menu_popup_trans_table;
  menu_trans_table = XtParseTranslationTable (menuTranslations);
  menu_popup_trans_table = XtParseTranslationTable (menupopupTranslations);

  i = 0;
  level++;

  while (menu[i].name != NULL)
    {
      switch (menu[i].type)
	{
	case BtnEle:
	  {
	    n = 0;
	    XtSetArg (args[n], XtNborderWidth, 0);
	    n++;
	    XtSetArg (args[n], XtNhighlightThickness, 1);
	    n++;
	    button = XtCreateManagedWidget (menu[i].name, menuButtonWidgetClass, parent, args, n);
	    XtOverrideTranslations (button, menu_trans_table);

	    n = 0;
	    XtSetArg (args[n], XtNmenuName, menu[i].name);
	    n++;
	    XtSetValues (button, args, n);
	    n = 0;
	    popupmenu[i] = XtCreatePopupShell (menu[i].name, simpleMenuWidgetClass, button, args, n);
	    XtOverrideTranslations (popupmenu[i], menu_popup_trans_table);

	    recursive_build (mb_obj, popupmenu[i], menu[i].popup);
	    break;
	  }
	case EntryEle:
	  {
	    entry = XtCreateManagedWidget (menu[i].name, smeBSBObjectClass, parent,
					   NULL, (Cardinal) 0);
	    XtAddCallback (entry, XtNcallback, menu[i].cb, NULL);
	    break;
	  }
	case SeparatorEle:
	  {
	    entry = XtCreateManagedWidget (menu[i].name, smeLineObjectClass, parent,
					   NULL, (Cardinal) 0);
	    break;
	  }
	}
      i++;
    }
  level--;
  return 0;
};

Menubar *
newmenubar (menubar)
     Widget menubar;
{
  Menubar *mb_obj;
  mb_obj = (Menubar *) absmalloc (sizeof (Menubar), "newmenubar:mb_obj ");

  level = 0;
  recursive_build (mb_obj, menubar, rootmenu);

  return mb_obj;
}
