/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#ifndef MENUBAR_H
#define MENUBAR_H
#include <stdio.h>
#include "param.h"
#include "cblist.h"

static char menuTranslations[] =
    "<EnterWindow>:     highlight()             \n\
     <LeaveWindow>:     reset()                 \n\
     <BtnDown>:         reset() PopupMenu()     ";

static char menupopupTranslations[] =
    "<EnterWindow>:     highlight()             \n\
     <LeaveWindow>:     unhighlight()           \n\
     <BtnMotion>:       highlight()             \n\
     <BtnUp>:           MenuPopdown() notify() unhighlight()";



typedef struct
{
 int Initialized;
 Widget btn1;
 Widget btn2;
 Widget btn3;
 Widget btn4;
 Widget btn5;
 Widget btn6;
 Widget popup1;
 Widget popup2;
 Widget popup3;
 Widget popup4;
 Widget popup5;
 Widget popup6;

}   Menubar;


extern Menubar *
newmenubar(Widget menubar);

extern Widget 
make_bar_button(Widget menu_bar, char *name);

extern Widget 
make_popup(Widget bar_button, char *name, char *button_name,
	   struct menuEntry * menuentry, int num);

extern Widget
make_toggle_pixmap(Widget parent, char *name, char **pix_bits, void (*ftccallback) (), XtPointer param);

extern Widget
make_button_pixmap(Widget parent, char *name, char **pix_bits, void (*ftccallback) (), XtPointer param);

extern void nocb();
extern void formatline();
extern void formatcolumn();
extern void m_win1();
extern void m_win2();
extern void m_win3();
extern void m_win4();
extern void m_win5();
extern void playmacro();

struct _Menu_ele
{
 char *name;
 int type;
 struct _Menu_ele *popup;
 void (*cb) ();
};

typedef struct _Menu_ele Menu_ele;

#define BtnEle 1
#define EntryEle 2
#define SeparatorEle 3


static Menu_ele filepopup[] = {
  {"New", EntryEle, NULL, m_nfile},
{			       "0pen...", EntryEle, NULL, X11m_ofile},
{			       "Close", EntryEle, NULL, m_close},
{			       "Separ1", SeparatorEle, NULL, m_close},
{			       "Save", EntryEle, NULL, X11m_sfile},
{			       "Save As...", EntryEle, NULL, X11m_sasfile},
{			       "Separ2", SeparatorEle, NULL, m_close},
{			       "Print", EntryEle, NULL, X11m_print},
{			       "Separ3", SeparatorEle, NULL, m_close},
{			       "Exit", EntryEle, NULL, m_exit},
{			       NULL, 0, NULL, NULL},
};




static Menu_ele editpopup[] = {
{			       "Cut", EntryEle, NULL, mcut},
{			       "Copy", EntryEle, NULL, mcpy},
{			       "Paste", EntryEle, NULL, mpst},
{			       "Separ4", SeparatorEle, NULL, m_close},
{			       "Copy Down", EntryEle, NULL, mcpyd},
{			       "Copy Right", EntryEle, NULL, mcpyr},
{			       NULL, 0, NULL, NULL},
};

static Menu_ele insertpopup[] = {
{                                 "Cells...", EntryEle, NULL, notyetcb},
{                                 "Line", EntryEle, NULL, m_insertline},
{                                 "Column", EntryEle, NULL, m_insertcolumn},
{                                 "Sheet", EntryEle, NULL, m_as},
{                                 "Function", EntryEle, NULL, X11m_formula},
{                                 NULL, 0, NULL, NULL},
};



static Menu_ele formatpopup[] = {
{				 "Cell...", EntryEle, NULL, X11m_formatnumber},
{				 "Line", EntryEle, NULL, formatline},
{				 "Column", EntryEle, NULL, formatcolumn},
{				 "Sheet", EntryEle, NULL, X11m_formatborder},
{				 NULL, 0, NULL, NULL},
};


static Menu_ele drawpopup[] = {
{			       "Line", EntryEle, NULL, m_plotline},
{			       "Rectangle", EntryEle, NULL, m_plotrect},
{			       "Arc", EntryEle, NULL, m_plotarc},
{			       "Ellipse", EntryEle, NULL, m_plotcircle},
{			       "Button", EntryEle, NULL, m_plotbutton},
{			       "->Edit On", EntryEle, NULL, m_editdraw},
{			       "->Edit Off", EntryEle, NULL, m_exitdraw},
{			       NULL, 0, NULL, NULL},
};

static Menu_ele chartpopup[] = {
{				"Create", EntryEle, NULL, m_creaplt},
{				"Option...", EntryEle, NULL, X11m_plotoption},
{				"Redraw", EntryEle, NULL,cb_redrawgra},
{				NULL, 0, NULL, NULL},
};

static Menu_ele toolpopup[] = {
{                                "Macro...", EntryEle, NULL, X11m_macro},
{                                "Insert Data File...", EntryEle, NULL, X11m_insertfl},
{                                "Export Data File...", EntryEle, NULL, X11m_exportfile},
{                                NULL, 0, NULL, NULL},
};

static Menu_ele windowpopup[] = {
{				 NULL, 0, NULL, NULL},
                                };

static Menu_ele helppopup[] = {
{                                "About...", EntryEle, NULL, m_about},
{                                "On line...", EntryEle, NULL, m_online},
{				 NULL, 0, NULL, NULL},
                                };

static Menu_ele rootmenu[] = {
{			      "File", BtnEle, filepopup, nocb},
{			      "Edit", BtnEle, editpopup, nocb},
{			      "Insert", BtnEle, insertpopup, nocb},
{			      "Format", BtnEle, formatpopup, nocb},
{			      "Draw", BtnEle, drawpopup, nocb},
{			      "Chart", BtnEle, chartpopup, nocb},
{			      "Tools", BtnEle, toolpopup, nocb},
{			      "Windows", BtnEle, windowpopup, nocb},
{			      "?", BtnEle, helppopup, nocb},
{			      NULL, 0, NULL, NULL},
};


#endif				/* MENUBAR_H */
