/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "module.h"
#include <string.h>
#include "memory.h"

static Module *ActiveModule = NULL;
#define AM ActiveModule

Module *
new_module ()
{
  Module *module = (Module *) absmalloc (sizeof (Module), "new_module:module");
  module->tree = NULL;
  module->nlc = 0;
  module->sourcecode = NULL;
  module->nentries = 0;
  module->entrylabel = NULL;
  module->entrynode = NULL;
  module->declared = 0;
  return module;
}

int
freemodule (module)
     Module *module;
{
  int i;

  if (module == NULL)
    return 1;
  if (module->tree != NULL)
    freenode (module->tree);

  for (i = 0; i < module->nlc; i++)
    {
      if (module->sourcecode[i] != NULL)
	absfree (module->sourcecode[i], "routine_freecode:nr->sourcecode[i] ");
    }
  if (module->sourcecode != NULL)
    absfree (module->sourcecode, "routine_freecode:nr->sourcecode ");

  if (module->entrynode != NULL)
    absfree (module->entrynode, "freemodule:module->entrynode");

  for (i = 0; i < module->nentries; i++)
    absfree (module->entrylabel[i], "freemodule:entrylabel[i]");

  if (module->entrylabel != NULL)
    absfree (module->entrylabel, "freemodule:module->:entrylabel");

  absfree (module, "freemodule:module");
  return 0;
}

int
module_freecode (module)
     Module *module;
{
  int i;

  if (module == NULL)
    return 1;

  if (module->tree != NULL)
    {
      freenode (module->tree);
      module->tree = NULL;
    }

  if (module->sourcecode != NULL)
    {
      for (i = 0; i < module->nlc; i++)
	{
	  if (module->sourcecode[i] != NULL)
	    absfree (module->sourcecode[i], "routine_freecode:nr->sourcecode[i] ");
	}
      absfree (module->sourcecode, "routine_freecode:nr->sourcecode ");
      module->sourcecode = NULL;
    }

  if (module->entrynode != NULL)
    {
      absfree (module->entrynode, "freemodule:module->entrynode");
      module->entrynode = NULL;
    }

  if (module->entrylabel != NULL)
    {
      for (i = 0; i < module->nentries; i++)
	absfree (module->entrylabel[i], "freemodule:entrylabel[i]");

      absfree (module->entrylabel, "freemodule:module->:entrylabel");
      module->entrylabel = NULL;
    }

  module->nlc = 0;
  module->nentries = 0;
  module->declared = 0;
  return 0;
}

int
module_nfile (module, filename)
     Module *module;
     char *filename;
{
  FILE *fp;
  int i;
  char line[258];
  module->nlc = 0;

  module_freecode (module);

  fp = fopen (filename, "r");
  while (fgets (line, 256, fp))
    module->nlc++;
  fclose (fp);

  if (module->nlc == 0)
    return 1;

  module->sourcecode = (char **) absmalloc
    (sizeof (char *) * (1 + module->nlc), "module_nfile:module->sourcecode ");
  if (!module->sourcecode)
    {
      fprintf (stderr, "Not enougth memory in module_nfile (dimsource = %d)\n", module->nlc);
      return 1;
    }

  fp = fopen (filename, "r");
  for (i = 0; i < module->nlc; i++)
    {
      fgets (line, 256, fp);
      module->sourcecode[i] = (char *) absmalloc (sizeof (char) * (strlen (line) + 3), "module_nfile:module->sourcecode[i] ");
      if (!module->sourcecode[i])
	{
	  fprintf (stderr, "Not enougth memory in module_nfile (line = %d)\n", i);
	  return 1;
	}
      strcpy (module->sourcecode[i], line);
      if (strchr (module->sourcecode[i], '\n') == NULL)
	strcat (module->sourcecode[i], "\n\0");
    }
  fclose (fp);
  module->sourcecode[module->nlc] = NULL;

  module->tree = parsefile (module->sourcecode);

  return 0;
}

Module *
ActivateModule (module)
     Module *module;
{
  ActiveModule = module;
  return ActiveModule;
};

int
module_declare (module)
     Module *module;
{
  ActiveModule = module;
  resetbreak ();
  if (!module->declared)
    exdecl (module->tree);
  module->declared = 1;
  return 0;
}

int
emitlabel (nodeType * p)
{
  if (AM == NULL)
    return 1;
  if (p->type == typeOpr)
    {
      {
	AM->entrynode = (nodeType **) absrealloc (AM->entrynode, (AM->nentries + 1) * sizeof (nodeType *), "emitlabel");
	AM->entrylabel = (char **) absrealloc (AM->entrylabel, (AM->nentries + 1) * sizeof (char *), "emitlabel");

	AM->entrynode[AM->nentries] = p;
	AM->entrylabel[AM->nentries] = (char *) absmalloc (
					       sizeof (char) * (1 + strlen (
			       ((obj *) (exint (p->opr.op[0]).rec.s))->label
								)
							    ), "emitlabel");
	strcpy (AM->entrylabel[AM->nentries], ((obj *) (exint (p->opr.op[0])).rec.s)->label);
	AM->nentries++;
      }
    }
  return 0;
}

int
module_s (Module * module, FILE * fp)
{
  int i = 0;
  int j = 0;
  char line[256];
  int skip = 0;

  if (module != NULL)
    while (module->sourcecode[i] != NULL)
      {
	if (strstr (module->sourcecode[i], "absmain") &&
	    strstr (module->sourcecode[i], "Sub"))
	  skip = 1;

	if (!skip)
	  {
	    if (!(module->sourcecode[i][0] == '\'' && module->sourcecode[i][1] == '='))
	      fprintf (fp, "%s", module->sourcecode[i]);
	  }
	else
	  {
	    strcpy (line, module->sourcecode[i]);
	    j = 0;
	    while (j < 256 && line[j] != '\0')
	      {
		if (line[j] >= 'A' && line[j] <= 'Z')
		  line[j] = 'a' + line[j] - 'A';
		j++;
	      }

	    if (strstr (line, "end") && strstr (line, "sub"))
	      skip = 0;
	  }

	i++;
      }
  return 0;

}
