/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "myscroll.h"
#include "worksheet.h"
#include "gr_interf.h"
#include "memory.h"
#include "callback.h"
#include <math.h>

#include "pixmaps/up.xpm"
#include "pixmaps/down.xpm"
#include "pixmaps/left.xpm"
#include "pixmaps/right.xpm"

static int nosetval = 0;
static int downonthumb = 0;

int
val2pos (vmin, vmax, h, val)
     int vmin, vmax, val, h;
{
  int pos = (val - vmin) * (h - 36) / (vmax - vmin) + 18;

  return pos;
}

int
pos2val (vmin, vmax, h, pos)
     int vmin, vmax, pos, h;
{
  int val = (vmax - vmin) * (pos - 18) / (h - 36) + vmin;

  return val;
}

int
redrawthumb (ms)
     Myscroll *ms;
{
  double vmin, val, vmax;
  double l, h;
  int x;

  vmin = ms->vmin;
  vmax = ms->vmax;
  l = ms->thumblen;
  val = ms->val;

  if (ms->orientation == horizontal)
    {
      h = ms->w - 32;
      x = h * ((val - vmin) / (vmax - vmin));
      desactivate_zm ();
      fillrectangle (XtDisplay (ms->back), XtWindow (ms->back), 3, 17, 2, ms->w - 32, ms->h - 4, 0, 0, 0);
      drawbuttonup (XtDisplay (ms->back), XtWindow (ms->back), x + 18, 3, x + 18 + ms->thumblen, 14, NULL, 0, 0);
      reactivate_zm ();
    }
  else
    {
      h = ms->h - 32;
      x = h * ((val - vmin) / (vmax - vmin));
      desactivate_zm ();
      fillrectangle (XtDisplay (ms->back), XtWindow (ms->back), 3, 2, 17, ms->w - 4, ms->h - 32, 0, 0, 0);
      drawbuttonup (XtDisplay (ms->back), XtWindow (ms->back), 3, x + 18, 14, x + 18 + ms->thumblen, NULL, 0, 0);
      reactivate_zm ();
    }
  return 0;
}

static int lstj1 = -1;
static int lstj2 = -1;
static int lsti1 = -1;
static int lsti2 = -1;

int
sethpos (ms, pos)
     Myscroll *ms;
     int pos;
{
  int i1, j1, i2, j2;

  if (pos < 18)
    pos = 18;
  if (pos > ms->w - ms->thumblen - 18)
    pos = ms->w - ms->thumblen - 18;
  ms->curx = pos;

  getvisi_cells (&i1, &j1, &i2, &j2);

  {
    double vmin, val, vmax;
    double x, l, h;
    vmin = 1;
    vmax = ActiveWorksheet->_mcol + 3;
    if (j2 > vmax)
      vmax = j2 + 3;
    h = ms->w - 32;
    val = j1;
    x = h * ((val - vmin) / (vmax - vmin));
    l = h * ((j2 - j1) / (vmax - vmin));

    j1 = x;
    j2 = l;

    if (!nosetval)
      {
	ms->vmin = vmin;
	if (downonthumb != -1 || vmax - 3 < ms->vmax)
	  {
	    ms->vmax = vmax - 3;
	    ms->thumblen = l;
	  }
      }
    if (j1 == lstj1 && j2 == lstj2 && downonthumb)
      return 0;
    lstj1 = j1;
    lstj2 = j2;
    desactivate_zm ();
    fillrectangle (XtDisplay (ms->back), XtWindow (ms->back), 3, 17, 2, ms->w - 32, ms->h - 4, 0, 0, 0);
    drawbuttonup (XtDisplay (ms->back), XtWindow (ms->back), j1 + 18, 3, j1 + 18 + ms->thumblen, 14, NULL, 0, 0);
    reactivate_zm ();
  }

  return 0;
}

int
setvpos (ms, pos)
     Myscroll *ms;
     int pos;
{
  int i1, j1, i2, j2;

  if (pos < 18)
    pos = 18;
  if (pos > ms->h - ms->thumblen - 18)
    pos = ms->h - ms->thumblen - 18;
  ms->cury = pos;

  getvisi_cells (&i1, &j1, &i2, &j2);

  {
    double vmin, val, vmax;
    double x, l, h;
    vmin = 1;
    vmax = ActiveWorksheet->mrow + 3;
    if (i2 > vmax)
      vmax = i2 + 3;
    h = ms->h - 32;
    val = i1;
    x = h * ((val - vmin) / (vmax - vmin));
    l = h * ((i2 - i1) / (vmax - vmin));

    i1 = x;
    i2 = l;

    if (!nosetval)
      {
	ms->vmin = vmin;
	if (downonthumb != -1 || vmax - 3 < ms->vmax)
	  {
	    ms->vmax = vmax - 3;
	    ms->thumblen = l;
	  }
      }
    if (i1 == lsti1 && i2 == lsti2 && downonthumb)
      return 0;
    lsti1 = i1;
    lsti2 = i2;

    desactivate_zm ();
    fillrectangle (XtDisplay (ms->back), XtWindow (ms->back), 3, 2, 17, ms->w - 4, ms->h - 32, 0, 0, 0);
    drawbuttonup (XtDisplay (ms->back), XtWindow (ms->back), 3, i1 + 18, 14, i1 + 18 + ms->thumblen, NULL, 0, 0);
    reactivate_zm ();
  }

  return 0;
}

void
resizescroll (widget, pointer, event, ctd)
     Widget widget;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int n;
  Dimension h, w;
  Myscroll *ms;
  Arg args[2];
  ms = (Myscroll *) pointer;

  n = 0;
  XtSetArg (args[n], XtNheight, &h);
  n++;
  XtSetArg (args[n], XtNwidth, &w);
  n++;
  XtGetValues (widget, args, n);

  ms->h = h;
  ms->w = w;

  if (h > w)
    {
      setvpos (ms, ms->cury);
      XtSetArg (args[0], XtNvertDistance, h - 16);
      XtSetValues (ms->down, args, 1);

    }
  else
    {
      sethpos (ms, ms->curx);
      XtSetArg (args[0], XtNhorizDistance, w - 16);
      XtSetValues (ms->right, args, 1);

    }
}

void
btncb (widget, pointer, junk)
     Widget widget;
     XtPointer pointer;
     XtPointer junk;
{

  int pos;

  Myscroll *ms = (Myscroll *) pointer;

  if (widget == ms->up || widget == ms->left)
    {
      ms->val -= 1;
      if (ms->val < ms->vmin)
	ms->val = ms->vmin;
    }
  if (widget == ms->down || widget == ms->right)
    {
      ms->val += 1;
      if (ms->val > ms->vmax)
	ms->val = ms->vmax;
    }
  if (widget == ms->thumb)
    {
    }

  if (ms->h > ms->w)
    {
      pos = val2pos (ms->vmin, ms->vmax, ms->h, ms->val);
      setvpos (ms, pos);
    }
  else
    {
      pos = val2pos (ms->vmin, ms->vmax, ms->w, ms->val);
      sethpos (ms, pos);
    }
  nosetval = 1;
  ms->cbfunction (ms->val);
  nosetval = 0;
};

void
thumbupcb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  Myscroll *ms = (Myscroll *) pointer;

  downonthumb = -1;

  if (ms->orientation == vertical)
    setvpos (ms, ms->cury);
  else
    sethpos (ms, ms->curx);
  downonthumb = 0;
}

void
movecb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int x, y;

  int oldval;
  Window root_return, child_return;
  int root_x_return, root_y_return;
  int win_x_return, win_y_return;
  unsigned int mask_return;
  Myscroll *ms = (Myscroll *) pointer;
  XQueryPointer (XtDisplay (w), XtWindow (w), &root_return, &child_return, &root_x_return,
		 &root_y_return, &win_x_return, &win_y_return, &mask_return);

  if (!downonthumb)
    return;

  x = root_x_return;
  y = root_y_return;

  ms->cury = y - ms->ssy;
  ms->curx = x - ms->ssx;

  if (ms->h > ms->w)
    {
      if (ms->cury < 18)
	ms->cury = 18;
      if (ms->cury > ms->h - ms->thumblen - 18)
	ms->cury = ms->h - ms->thumblen - 18;
      oldval = ms->val;
      ms->val = pos2val (ms->vmin, ms->vmax, ms->h, ms->cury);
      if (oldval == ms->val)
	return;
      redrawthumb (ms);
    }
  else
    {
      if (ms->curx < 18)
	ms->curx = 18;
      if (ms->curx > ms->w - 18 - ms->thumblen)
	ms->curx = ms->w - 18 - ms->thumblen;
      oldval = ms->val;
      ms->val = pos2val (ms->vmin, ms->vmax, ms->w, ms->curx);
      if (oldval == ms->val)
	return;
      redrawthumb (ms);
    }
  nosetval = 1;
  ms->cbfunction (ms->val);
  nosetval = 0;
};

void
stepcb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int x, y;

  double delta;

  Myscroll *ms = (Myscroll *) pointer;

  x = event->xbutton.x;
  y = event->xbutton.y;

  if ((ms->orientation == horizontal && x > ms->curx && x < ms->curx + ms->thumblen) ||
      (ms->orientation == vertical && y > ms->cury && y < ms->cury + ms->thumblen))
    {

      downonthumb = 1;
      ms->ssx = -ms->curx + event->xbutton.x_root;
      ms->ssy = -ms->cury + event->xbutton.y_root;
      return;
    }

  if (ms->h > ms->w)
    {
      if (y < 18 || y > ms->h - ms->thumblen - 18)
	return;

      if (fabs (y - ms->cury) < 10)
	ms->cury = y;
      else
	{
	  delta = (y - ms->cury) / 3.0;
	  ms->cury += delta;
	}
      if (ms->cury < 18)
	ms->cury = 18;
      if (ms->cury > ms->h - ms->thumblen - 18)
	ms->cury = ms->h - ms->thumblen - 18;

      ms->val = pos2val (ms->vmin, ms->vmax, ms->h, ms->cury);
      setvpos (ms, ms->cury);

    }
  else
    {
      if (x < 18 || x > ms->w - 18 - ms->thumblen)
	return;

      if (fabs (x - ms->curx) < 10)
	ms->curx = x;
      else
	{
	  delta = (x - ms->curx) / 3.0;
	  ms->curx += delta;
	}
      if (ms->curx < 18)
	ms->curx = 18;
      if (ms->curx > ms->w - ms->thumblen - 18)
	ms->curx = ms->w - ms->thumblen - 18;

      ms->val = pos2val (ms->vmin, ms->vmax, ms->w, ms->curx);
      sethpos (ms, ms->curx);

    }
  nosetval = 1;
  ms->cbfunction (ms->val);
  nosetval = 0;

};

void
presscb (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int x, y;

  Myscroll *ms = (Myscroll *) pointer;

  x = event->xbutton.x_root;
  y = event->xbutton.y_root;

  ms->thumbpy = y;
  ms->thumbpx = x;
  ms->ssx = ms->curx;
  ms->ssy = ms->cury;
};

int
setscrolllimit (ms, vmin, vmax)
     Myscroll *ms;
     int vmin, vmax;
{
  ms->vmin = vmin;
  ms->vmax = vmax;

  return 0;
};

void
setscrollval (ms, val)
     Myscroll *ms;
     int val;
{

  if (nosetval)
    return;

  ms->val = val;
  if (ms->val < ms->vmin)
    ms->val = ms->vmin;
  if (ms->val > ms->vmax)
    ms->val = ms->vmax;

  if (ms->h > ms->w)
    {

      ms->cury = val2pos (ms->vmin, ms->vmax, ms->h, ms->val);
      setvpos (ms, ms->cury);

    }
  else
    {
      ms->curx = val2pos (ms->vmin, ms->vmax, ms->w, ms->val);
      sethpos (ms, ms->curx);

    }

}

Myscroll *
CreateMyscroll (parent, h, w, vmin, vmax, func)
     Widget parent;
     int h, w, vmin, vmax;
     void (*func) ();
{

  Arg args[16];
  int n;
  Myscroll *ms;

  ms = (Myscroll *) absmalloc (sizeof (Myscroll), "CreateMyscroll:ms ");

  ms->parent = parent;
  ms->cbfunction = func;
  ms->h = h;
  ms->w = w;
  ms->vmin = vmin;
  ms->vmax = vmax;
  ms->val = 0;
  ms->thumblen = 30;

  XtSetArg (w_args[w_n], XtNheight, h);
  w_n++;
  XtSetArg (w_args[w_n], XtNwidth, w);
  w_n++;
  XtSetArg (w_args[w_n], XtNborderWidth, 0);
  w_n++;
  ms->back = (Widget) XtCreateManagedWidget ("window", formWidgetClass, parent,
					     w_args, w_n);
  if (h > w)
    {
      ms->orientation = vertical;
      ms->up = (Widget) make_repeater_pixmap (ms->back, "up", up_xpm, btncb, (XtPointer) ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->up, args, n);

      ms->cury = 18;

      ms->down = (Widget) make_repeater_pixmap (ms->back, "down", down_xpm, btncb, (XtPointer) ms);
      n = 0;
      XtSetArg (args[n], XtNfromVert, (XtPointer) NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, h - 14);
      n++;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetValues (ms->down, args, n);
    }
  else
    {
      ms->orientation = horizontal;
      ms->left = (Widget) make_repeater_pixmap (ms->back, "left", left_xpm, btncb, (XtPointer) ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, 1);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->left, args, n);

      ms->curx = 18;

      ms->right = (Widget) make_repeater_pixmap (ms->back, "right", right_xpm, btncb, (XtPointer)
						 ms);
      n = 0;
      XtSetArg (args[n], XtNfromHoriz, NULL);
      n++;
      XtSetArg (args[n], XtNhorizDistance, w - 14);
      n++;
      XtSetArg (args[n], XtNfromVert, NULL);
      n++;
      XtSetArg (args[n], XtNvertDistance, 1);
      n++;
      XtSetValues (ms->right, args, n);

    }

  XtAddEventHandler (ms->back, ButtonPressMask, False, stepcb, (XtPointer) ms);
  XtAddEventHandler (ms->back, ExposureMask, FALSE, shadow2, (XtPointer) ms->back);
  XtAddEventHandler (ms->back, ButtonReleaseMask, False, thumbupcb, (XtPointer) ms);

  XtAddEventHandler (ms->back, Button1MotionMask, False, movecb, (XtPointer) ms);

  XtAddEventHandler (ms->back, StructureNotifyMask, False, resizescroll, (XtPointer) ms);
  return ms;
}
