/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#ifndef NODE_H
#define NODE_H
#include "object.h"


typedef enum { typeCon, typeId, typeOpr ,typeOpr1, typeOpr2 ,typeMember} nodeEnum;

/* constants */
typedef struct {
    nodeEnum type;              /* type of node */
    obj value;                  /* value of constant */
} conNodeType;

/* identifiers */
typedef struct {
    nodeEnum type;              /* type of node */
    obj id;                      /* subscript to ident matrix */
} idNodeType;

/* member */
typedef struct {
    nodeEnum type;              /* type of node */
    obj member;                      /* subscript to ident matrix */
} memberNodeType;

/* operators */
typedef struct {
    nodeEnum type;              /* type of node */
    obj oper;                   /* operator */
    int nops;                   /* number of operands */
    union nodeTypeTag **op;   /* operands (expandable) */
} oprNodeType;

typedef struct {
    nodeEnum type;              /* type of node */
    obj oper;                   /* operator */
    union nodeTypeTag *op1;   /* operands (expandable) */
    union nodeTypeTag *op2;   /* operands (expandable) */
} opr2NodeType;

typedef struct {
    nodeEnum type;              /* type of node */
    obj oper;                   /* operator */
    union nodeTypeTag *op1;   /* operands (expandable) */
} opr1NodeType;

typedef union nodeTypeTag {
    nodeEnum type;              /* type of node */
    conNodeType con;            /* constants */
    idNodeType id;              /* identifiers */
    oprNodeType opr;            /* operators */
    opr1NodeType opr1;            /* operators */
    opr2NodeType opr2;            /* operators */
    memberNodeType member;
} nodeType;

nodeType * opr(int oper,char*label,int nops, ...);
nodeType * opr1(int oper,char*label,nodeType *p1);
nodeType * opr2(int oper,char*label,nodeType *p1,nodeType *p2);
nodeType * addarg(nodeType *p1,nodeType *p2);
nodeType * id(obj i);
nodeType * member(obj i);
nodeType * con(obj value);
void freenode(nodeType *p);
void printTree(nodeType *p,int deep);
#endif /* NODE_H*/
