/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "module.h"
#include "project.h"
#include "memory.h"
#include "string.h"
#include "symboltable.h"

static Project *ActiveProject = NULL;
#define AP ActiveProject

int
getnmacro (project)
     Project *project;
{
  if (project->nmodule > 0)
    return project->modules[0]->nentries;
  return 0;
}

char *
getmacroname (project, v)
     Project *project;
     int v;
{
  if (v >= 0 && v < project->modules[0]->nentries)
    return project->modules[0]->entrylabel[v];
  return "bad macro num";
}

void
playmacro (project, pos)
     Project *project;
     int pos;
{
  ActivateProject (project);

  gotolabel (project->modules[0]->entrylabel[pos]);

  xrepnt ();
};

void
editmacro (project, pos)
     Project *project;
     int pos;
{
  FILE *tmpmac;
  int i;

  if (project == NULL)
    return;

  mainwin_popup_abv ();
  tmpmac = fopen ("abstmpmacro", "w");

  if (project->nmodule < 1)
    project_addmodule (project, NULL);
  else
    {
      i = 0;
      if (project->modules[0]->sourcecode != NULL)
	while (project->modules[0]->sourcecode[i] != NULL)
	  {
	    fprintf (tmpmac, "%s", project->modules[0]->sourcecode[i]);
	    i++;
	  }
    }
  fclose (tmpmac);

  xedit_nfile ("abstmpmacro");

  return;
}

void
updatemodule ()
{
  if (ActiveProject == NULL)
    return;
  module_nfile (ActiveProject->modules[0], "abstmpmacro");
}

Project *
new_project ()
{
  Project *project = (Project *) absmalloc (sizeof (Project), "new_project:project");
  project->modules = NULL;
  project->nmodule = 0;
  ActiveProject = project;
  return project;
}

int
freeproject (project)
     Project *project;
{
  int i;
  if (project == NULL)
    return 1;
  for (i = 0; i < project->nmodule; i++)
    freemodule (project->modules[i]);

  if (project->modules != NULL)
    absfree (project->modules, "freeproject:project->modules");

  absfree (project, "freeproject:project");
  return 0;

};

int
project_s (Project * project, FILE * fp)
{
  int i;
  fprintf (fp, "'===================ABV PROJECT======================\n");
  for (i = 0; i < project->nmodule; i++)
    {
      fprintf (fp, "'===================MODULE %d  =======================\n", i + 1);
      module_s (project->modules[i], fp);
    }
  return 0;
}

Project *
ActivateProject (project)
     Project *project;
{
  ActiveProject = project;
  if (project->nmodule > 0)
    ActivateModule (project->modules[0]);
  return ActiveProject;
};

int
project_addmodule (project, filename)
     Project *project;
     char *filename;
{
  project->nmodule++;
  project->modules = (Module **) absrealloc (project->modules, sizeof (Module *) * project->nmodule, "projet_addmodule:project->modules");
  project->modules[project->nmodule - 1] = new_module ();
  ActivateModule (project->modules[project->nmodule - 1]);

  if (filename != NULL)
    module_nfile (project->modules[project->nmodule - 1], filename);
  return 0;
}

int
gotolabel (char *buf)
{
  int i, j;
  resetbreak ();
  for (j = 0; j < AP->nmodule; j++)
    for (i = 0; i < AP->modules[j]->nentries; i++)
      {
	if (strcasecmp (buf, AP->modules[j]->entrylabel[i]) == 0)
	  {
	    module_declare (AP->modules[j]);
	    newlocalidtable ();

	    exint (AP->modules[j]->entrynode[i]);
	    deletelocalidtable ();

	    return 0;
	  }

      }
  return -1;
}

int
labelexist (Project * project, char *buf)
{
  int i, j;

  for (j = 0; j < project->nmodule; j++)
    for (i = 0; i < project->modules[j]->nentries; i++)
      {
	if (strcasecmp (buf, project->modules[j]->entrylabel[i]) == 0)
	  {
	    return 1;
	  }

      }
  return 0;
}
