/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#include "symboltable.h"
#include "defid.h"
#include "memory.h"

#define BIGTABLESIZE 100
bigitem *bigtable[BIGTABLESIZE];

char *
biglook (const char *p, int ins)
{
  int ii = 0;
  int pos = 0;
  bigitem *n;
  bigitem *nn;
  const char *pp = p;

  if (p == NULL)
    return NULL;

  while (*pp)
    ii = ii << 1 ^ *pp++;
  if (ii < 0)
    ii = -ii;
  ii %= BIGTABLESIZE;

  for (n = bigtable[ii]; n; n = n->next)
    {
      if (p != NULL && n->label != NULL)
	if (strcasecmp (p, n->label) == 0)
	  {
	    if (ins < 0)
	      {
		n->ref--;
		if (n->ref <= 0)
		  {
		    free (n->label);

		  }
	      }
	    else
	      {

		n->ref++;
		return n->label;
	      }
	  }
      pos++;
    }

  if (ins == 0)
    fprintf (stderr, "label %s not found!\n", p);

  nn = (bigitem *) absmalloc (sizeof (bigitem), "biglook:nn");
  nn->label = (char *) absmalloc (strlen (p) + 1, "biglook:nn->label");
  if (nn->label == NULL)
    fprintf (stderr, "out of memory in bigtable!!!!\n");
  strcpy (nn->label, p);
  nn->next = bigtable[ii];
  bigtable[ii] = nn;
  nn->ref = 1;
  return nn->label;
}

#define TBLSZ  23
#define MAXDEEP 10
name *table[TBLSZ];

name *localtable[MAXDEEP][TBLSZ];
static int deep = -1;

int
newlocalidtable ()
{
  deep++;
  return deep;
};
int
deletelocalidtable ()
{
  deep--;
  return deep;
};

obj *
look (const char *p, int ins)
{
  int ii = 0;
  name *n;
  name *nn;
  const char *pp = p;

  if (p == NULL)
    return NULL;

  while (*pp)
    ii = ii << 1 ^ *pp++;
  if (ii < 0)
    ii = -ii;
  ii %= TBLSZ;

  if (deep >= 0 && deep < MAXDEEP)
    for (n = localtable[deep][ii]; n; n = n->next)
      {
	if (p != NULL && n->label != NULL)
	  if (strcasecmp (p, n->label) == 0)
	    return &(n->id);
      }

  for (n = table[ii]; n; n = n->next)
    {
      if (p != NULL && n->label != NULL)
	if (strcasecmp (p, n->label) == 0)
	  return &(n->id);
    }

  if (ins == 0)
    fprintf (stderr, "name not found\n");

  if (deep >= 0 && deep < MAXDEEP)
    {
      nn = (name *) absmalloc (sizeof (name), "look:nn(local)");
      nn->label = biglook (p, 1);
      nn->next = localtable[deep][ii];
      localtable[deep][ii] = nn;
      nn->id.type = -1;
      nn->id.label = nn->label;
      return &(nn->id);

    }

  nn = (name *) absmalloc (sizeof (name), "look:nn(global)");
  nn->label = biglook (p, 1);
  nn->next = table[ii];
  table[ii] = nn;
  nn->id.type = -1;
  nn->id.label = nn->label;
  return &(nn->id);
}

obj *
insert (const char *s)
{
  return look (s, 1);
}

int
inittables ()
{
  int i, j, numid;
  obj *o;

  for (i = 0; i < MAXDEEP; i++)
    for (j = 0; j < TBLSZ; j++)
      localtable[i][j] = NULL;

  for (j = 0; j < TBLSZ; j++)
    table[j] = NULL;

  for (j = 0; j < BIGTABLESIZE; j++)
    bigtable[j] = NULL;

  numid = 0;
  while (defaultid[numid].name != NULL)
    {
      o = look (defaultid[numid].name, 1);
      o->type = defaultid[numid].type;
      switch (o->type)
	{
	case INTEGER_CONSTANT:
	  o->rec.i = defaultid[numid].i;
	  break;
	case FLOATING_CONSTANT:
	  o->rec.d = defaultid[numid].val;
	  break;
	case STRING_CONSTANT:
	  o->rec.s = defaultid[numid].str;
	  break;
	}
      numid++;
    }
  return 0;
}
