/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "widget_makerX11.h"
#include "info_dialog.h"
#include "inputbox.h"
#include "workbook.h"
#include "mainwin.h"
#include "left.xpm"
#include "right.xpm"
#include "first.xpm"
#include "last.xpm"
#include "check0.xpm"
#include "check1.xpm"

#include "callback.h"
#include "gr_interf.h"

static struct menuEntry popupMenuEntries[] =
{
  {"Cut", mcut},
  {"Copy", mcpy},
  {"Paste", mpst},
  {"Copy Down", mcpyd},
  {"Copy Right", mcpyr},
};

Pixmap
make_bitmap_from_data (parent, bits, w, h)
     Widget parent;
     char *bits;
     int w, h;
{
  return
    XCreateBitmapFromData (XtDisplay (parent),
			DefaultRootWindow (XtDisplay (parent)), bits, w, h);
}

int
make_pixmap_from_data (parent, bits, bm)
     Widget parent;
     char **bits;
     Pixmap *bm;
{
  XpmAttributes xpm_attr;
  XpmColorSymbol none_color =
  {NULL, "None", (Pixel) 0};

  xpm_attr.valuemask = XpmSize | XpmCloseness;
  xpm_attr.closeness = 40000;
  xpm_attr.numsymbols = 1;
  xpm_attr.colorsymbols = &none_color;
  XtVaGetValues (parent, XtNbackground, &none_color.pixel, NULL);
  XpmCreatePixmapFromData (XtDisplay (parent),
	 DefaultRootWindow (XtDisplay (parent)), bits, bm, NULL, &xpm_attr);

  return 0;
}

Widget
make_button_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  if (ftccallback != NULL)
    {
      XtSetArg (args[n], XtNhighlightThickness, 1);
      n++;
      XtSetArg (args[n], XtNborderWidth, 0);
      n++;
      XtSetArg (args[n], XtNinternalHeight, 1);
      n++;
      XtSetArg (args[n], XtNinternalWidth, 1);
      n++;
    }
  else
    {
      XtSetArg (args[n], XtNhighlightThickness, 0);
      n++;
      XtSetArg (args[n], XtNborderWidth, 0);
      n++;
      XtSetArg (args[n], XtNinternalHeight, 1);
      n++;
      XtSetArg (args[n], XtNinternalWidth, 1);
      n++;
    }
  button = XtCreateManagedWidget (name, commandWidgetClass, parent, args, n);
  if (ftccallback != NULL)
    XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_bar_button_pixmap (menu_bar, name, pix_bits)
     Widget menu_bar;
     char *name;
     char **pix_bits;
{
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (menu_bar, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  return XtCreateManagedWidget
    (name, menuButtonWidgetClass, menu_bar, args, n);
}

Widget
make_repeater_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  button = XtCreateManagedWidget (name, repeaterWidgetClass, parent, args, n);
  XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_toggle_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  button = XtCreateManagedWidget (name, toggleWidgetClass, parent, args, n);
  if (ftccallback != NULL)
    XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_menu_button_pixmap (parent, name, pix_bits, ftccallback, param)
     Widget parent;
     char *name;
     char **pix_bits;
     void (*ftccallback) ();
     XtPointer param;
{
  Pixmap bm;
  Arg args[20];
  register int n;

  n = 0;
  make_pixmap_from_data (parent, pix_bits, &bm);
  XtSetArg (args[n], XtNbitmap, bm);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 1);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 1);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 1);
  n++;
  return XtCreateManagedWidget (name, menuButtonWidgetClass, parent, args, n);

}

static first_check = 1;
static Pixmap check0;
static Pixmap check1;

void
changecheck (w, closure, call_data)
     Widget w;
     XtPointer closure, call_data;
{
  Arg args[1];
  Boolean on;

  XtSetArg (args[0], XtNstate, &on);
  XtGetValues (w, args, 1);

  if (on)
    XtSetArg (args[0], XtNbitmap, check1);
  else
    XtSetArg (args[0], XtNbitmap, check0);

  XtSetValues (w, args, 1);
}

int
setcheck (w, on)
     Widget w;
     int on;
{
  Arg args[2];

  XtSetArg (args[0], XtNstate, on);

  if (on)
    XtSetArg (args[1], XtNbitmap, check1);
  else
    XtSetArg (args[1], XtNbitmap, check0);

  XtSetValues (w, args, 2);
  return on;
}

Boolean
getcheck (w)
     Widget w;
{
  Boolean on;
  Arg args[1];
  XtSetArg (args[0], XtNstate, &on);
  XtGetValues (w, args, 1);
  return on;
}

Widget
make_check (parent, name, ftccallback, param)
     Widget parent;
     char *name;
     void (*ftccallback) ();
     XtPointer param;
{
  Widget button;
  Arg args[20];
  register int n;

  if (first_check)
    {
      make_pixmap_from_data (parent, check0_xpm, &check0);
      make_pixmap_from_data (parent, check1_xpm, &check1);
      first_check = 0;
    }

  n = 0;
  XtSetArg (args[n], XtNbitmap, check0);
  n++;
  XtSetArg (args[n], XtNlabel, "");
  n++;

  XtSetArg (args[n], XtNhighlightThickness, 0);
  n++;
  XtSetArg (args[n], XtNborderWidth, 0);
  n++;
  XtSetArg (args[n], XtNinternalHeight, 0);
  n++;
  XtSetArg (args[n], XtNinternalWidth, 0);
  n++;
  button = XtCreateManagedWidget (name, toggleWidgetClass, parent, args, n);

  XtAddCallback (button, XtNcallback, changecheck, NULL);

  if (ftccallback != NULL)
    XtAddCallback (button, XtNcallback, ftccallback, param);
  return button;
}

Widget
make_alone_popup (_toplevel, name)
     Widget _toplevel;
     char *name;
{
  Arg args[20];
  register int n;
  Widget popupmenu, entry;
  int i;
  n = 0;

  XtSetArg (args[n], XtNinput, True);
  n++;

  popupmenu = XtCreatePopupShell
    ("popupMenu", simpleMenuWidgetClass, _toplevel, args, n);

  for (i = 0; i < XtNumber (popupMenuEntries); i++)
    {
      entry = XtCreateManagedWidget (popupMenuEntries[i].name,
			  smeBSBObjectClass, popupmenu, NULL, (Cardinal) 0);
      XtAddCallback (entry, XtNcallback, popupMenuEntries[i].function, NULL);
    }

  return popupmenu;
}

void
changesheetname (w, pointer, event, ctd)
     Widget w;
     XtPointer pointer;
     XEvent *event;
     Boolean *ctd;
{
  int s = 0;
  int sheet;
  char *value;
  int i;

  if (((XButtonEvent *) event)->button == Button1)
    {
      int s = 0;
      for (s = 0; s < 20; s++)
	if (w == ActiveMainwin->selectsheet[s])
	  cbcgsht (s + ActiveWorkbook->first_vis_wks);
      return;
    }

  if (((XButtonEvent *) event)->button != Button3)
    return;

  for (s = 0; s < 20; s++)
    if (w == ActiveMainwin->selectsheet[s])
      sheet = s + ActiveWorkbook->first_vis_wks;
  value = inputbox ("New worksheet name:", "worksheet name", ActiveWorkbook->Worksheets[sheet]->Name);
  if (value != NULL)
    if (strlen (value) > 0)
      {
	for (i = 0; i < ActiveWorkbook->nbworksheet; i++)
	  {
	    if (strcasecmp (ActiveWorkbook->Worksheets[i]->Name, value) == 0)
	      {
		Inform ("Worksheet name allready in use!", NULL);
		return;
	      }
	  }

	worksheet_setname (ActiveWorkbook->Worksheets[sheet], value);
      }
}

static Widget thumbpar;

void
build_thumb (parent)
     Widget parent;
{
  int i;
  int width;
  int pos = 0;
  char *name = "?";
  ActiveMainwin->first = make_button_pixmap (ActiveMainwin->sheetbtn, "first", first_xpm,
					     cb_first_sheet, NULL);
  w_n = 0;
  w_rel (NULL, 0, 0);
  w_set (ActiveMainwin->first);

  ActiveMainwin->previous = make_button_pixmap (ActiveMainwin->sheetbtn, "previous", left_xpm, cb_previous_sheet, NULL);
  w_n = 0;
  w_relv (NULL, 0);
  w_relh (ActiveMainwin->first, 0);
  w_set (ActiveMainwin->previous);

  ActiveMainwin->next = make_button_pixmap (ActiveMainwin->sheetbtn, "next", right_xpm,
					    cb_next_sheet, NULL);
  w_n = 0;
  w_relh (ActiveMainwin->previous, 0);
  w_relv (NULL, 0);
  w_set (ActiveMainwin->next);

  ActiveMainwin->last = make_button_pixmap (ActiveMainwin->sheetbtn, "last", last_xpm,
					    cb_last_sheet, NULL);
  w_n = 0;
  w_relh (ActiveMainwin->next, 0);
  w_relv (NULL, 0);
  w_set (ActiveMainwin->last);

  thumbpar = parent;

  for (i = 0; i < 20; i++)
    {
      width = 20;
      w_n = 0;
      w_relv (NULL, 0);
      w_relh (NULL, pos);

      w_dim (width + 15, 20);
      w_bord (0);
      pos += width + 15;
      XtSetArg (w_args[w_n], XtNbackground, absblack);
      w_n++;
      XtSetArg (w_args[w_n], XtNforeground, abswhite);
      w_n++;

      if (i == 0)
	XtSetArg (w_args[w_n], XtNstate, True);
      else
	XtSetArg (w_args[w_n], XtNstate, False);
      w_n++;
      if (i > 0)
	{
	  XtSetArg (w_args[w_n], XtNradioGroup, ActiveMainwin->selectsheet[0]);
	  w_n++;
	}
      ActiveMainwin->selectsheet[i] = XtCreateWidget (name, toggleWidgetClass, parent, w_args, w_n);

      XtAddEventHandler (ActiveMainwin->selectsheet[i], ButtonReleaseMask, True, changesheetname, ActiveMainwin->selectsheet[i]);

    }
  return;
}

int
rebuild_thumb ()
{
  int i, j;
  int width;
  int nb;
  int pos = 0;
  char *name;

  if (ActiveWorkbook == NULL)
    return -1;

  for (i = 0; i < 20; i++)
    if (ActiveMainwin->selectsheet[i] != NULL)
      {
	XtUnmanageChild (ActiveMainwin->selectsheet[i]);
      }
  nb = ActiveWorkbook->nbworksheet;
  j = 0;
  for (i = ActiveWorkbook->first_vis_wks; i < nb && j < 19; i++)
    {
      name = ActiveWorkbook->Worksheets[i]->Name;
      if (name == NULL)
	name = "?";
      width = gettextw (name, strlen (name), 1, 0, 2) + 25;
      w_n = 0;
      w_relh (NULL, pos);
      w_dim (width, 20);
      w_arg (XtNlabel, name);

      pos += width;
      if (ActiveWorkbook->Worksheets[i] == ActiveWorkbook->activeworksheet)
	XtSetArg (w_args[w_n], XtNstate, True);
      else
	XtSetArg (w_args[w_n], XtNstate, False);
      w_n++;

      w_set (ActiveMainwin->selectsheet[j]);
      XtManageChild (ActiveMainwin->selectsheet[j]);
      j++;
    }
  return 0;
}
