/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/


#include "worksheet.h"
#include "workbook.h"
#include "memory.h"
#include "newplot.h"
#include "main.h"
#include "callback_decl.h"

Worksheet *ActiveWorksheet;
extern void setscrollvlimit (int vmin, int vmax);
extern void setscrollhlimit (int vmin, int vmax);
extern void setscrollv (double v);
extern void setscrollh (double v);
extern void setzmlabel (int z);
extern int rebuild_thumb ();

static int nworksheet = 1;

Worksheet *
newworksheet ()
{
  int i;
  Worksheet *worksheet;
  worksheet = (Worksheet *) absmalloc (sizeof (Worksheet), "newworksheet:worksheet ");

  if (worksheet == NULL)
    fprintf (stderr, "worksheet not allocated!");

  worksheet->nlin = NBLIN;
  worksheet->ncol = NBCOL;
  worksheet->cw[0] = -WCOL;
  worksheet->lh[0] = -HLIN;
  for (i = 1; i <= NBCOL; i++)
    worksheet->cw[i] = worksheet->cw[i - 1] + WCOL;
  for (i = 1; i <= NBLIN; i++)
    worksheet->lh[i] = worksheet->lh[i - 1] + HLIN;

  worksheet->ll = 1;
  worksheet->cc = 1;
  worksheet->nbcell = 0;
  worksheet->nbchart = 0;
  worksheet->nbdrawing = 0;
  worksheet->nbbutton = 0;
  worksheet->zm = 100;

  worksheet->Graphs = NULL;
  worksheet->Drawings = NULL;
  worksheet->Buttons = NULL;

  worksheet->selection = newselection ();

  worksheet->defcell = newcell (-1, -1);

  ActiveCell = worksheet->defcell;
  ActiveGraph = NULL;
  ActiveDrawing = NULL;
  ActiveButton = NULL;

  worksheet->activecell = ActiveCell;
  worksheet->activechart = NULL;
  worksheet->activedrawing = NULL;
  worksheet->activebutton = NULL;

  ActiveSelection = worksheet->selection;

  worksheet->objn = nworksheet++;

  worksheet->matrix = NULL;
  worksheet->mrow = -1;
  worksheet->_mcol = 0;
  worksheet->mcol = NULL;

  return worksheet;
}

Worksheet *
newworksheet1 (nbcell, nbchart, nbdrawing)
     int nbcell, nbchart, nbdrawing;
{
  int i;
  Worksheet *worksheet;
  worksheet = (Worksheet *) absmalloc (sizeof (Worksheet), "newworksheet1:worksheet ");

  worksheet->nlin = NBLIN;
  worksheet->ncol = NBCOL;
  worksheet->cw[0] = -WCOL;
  worksheet->lh[0] = -HLIN;
  for (i = 1; i <= NBCOL; i++)
    worksheet->cw[i] = worksheet->cw[i - 1] + WCOL;
  for (i = 1; i <= NBLIN; i++)
    worksheet->lh[i] = worksheet->lh[i - 1] + HLIN;

  worksheet->ll = 1;
  worksheet->cc = 1;
  worksheet->nbcell = nbcell;
  worksheet->nbchart = nbchart;
  worksheet->nbdrawing = nbdrawing;
  worksheet->nbbutton = 0;
  worksheet->zm = 100;

  worksheet->Graphs = NULL;
  worksheet->Drawings = NULL;
  worksheet->Buttons = NULL;

  worksheet->defcell = newcell (-1, -1);
  ActiveCell = worksheet->defcell;

  ActiveGraph = worksheet->Graphs[0];
  ActiveDrawing = worksheet->Drawings[0];
  worksheet->selection = newselection ();
  worksheet->activecell = ActiveCell;
  worksheet->activechart = NULL;
  worksheet->activedrawing = NULL;
  worksheet->activebutton = NULL;

  if (worksheet == NULL)
    printf ("not allocated!");

  worksheet->objn = nworksheet++;

  worksheet->matrix = NULL;
  worksheet->mrow = -1;
  worksheet->_mcol = 0;
  worksheet->mcol = NULL;
  return worksheet;
}

int
freeworksheet (worksheet)
     Worksheet *worksheet;
{
  int i, j;

  if (worksheet == NULL)
    return 1;

  if (ActiveWorksheet == worksheet)
    {
      for (i = 0; i < ActiveWorksheet->nbchart; i++)
	deletechartpixmap (ActiveWorksheet->Graphs[i]);
      ActiveWorksheet = NULL;
    }

  for (i = 0; i < worksheet->nbchart; i++)
    freechart (worksheet->Graphs[i]);
  for (i = 0; i < worksheet->nbdrawing; i++)
    freedrawing (worksheet->Drawings[i]);
  for (i = 0; i < worksheet->nbbutton; i++)
    freebutton (worksheet->Buttons[i]);

  freeselection (worksheet->selection);

  for (i = 1; i <= worksheet->mrow; i++)
    {
      for (j = 1; j <= worksheet->mcol[i]; j++)
	freecell (worksheet->matrix[i][j]);
      absfree (worksheet->matrix[i], "freeworksheet:worksheet->matrix[i] ");
    }
  absfree (worksheet->matrix, "freeworksheet:worksheet->matrix ");
  absfree (worksheet->mcol, "freeworksheet:worksheet->mcol ");

  freecell (worksheet->defcell);

  absfree (worksheet, "freeworksheet:worksheet ");

  return 0;
}

int
worksheet_w (worksheet, fp)
     Worksheet *worksheet;
     FILE *fp;
{
  int i, j;
  char buf[10];
  double v;

  for (i = 0; i < worksheet->ncol - 1; i++)
    if ((worksheet->cw[i + 1] - worksheet->cw[i]) != WCOL)
      {
	coltoalpha (buf, i);
	v = (worksheet->cw[i + 1] - worksheet->cw[i]) / 7.;
	fprintf (fp, "Columns(\"%s\").ColumnWidth = %f\n", buf, v);
      }

  for (i = 0; i < worksheet->nlin - 1; i++)
    if ((worksheet->lh[i + 1] - worksheet->lh[i]) != HLIN)
      {
	fprintf (fp, "Rows(\"%d\").RowHeight = %d\n", i, worksheet->lh[i + 1] - worksheet->lh[i]);
      }

  for (i = 1; i <= worksheet->mrow; i++)
    for (j = 1; j <= worksheet->mcol[i]; j++)
      {
	if (worksheet->matrix[i][j] != NULL)
	  cell_w (worksheet->matrix[i][j], fp);
      }

  for (i = 0; i < worksheet->nbchart; i++)
    chart_w (worksheet->Graphs[i], fp);
  for (i = 0; i < worksheet->nbdrawing; i++)
    drawing_w (worksheet->Drawings[i], fp);
  for (i = 0; i < worksheet->nbbutton; i++)
    button_w (worksheet->Buttons[i], fp);

  return 0;
}

int
worksheet_update (worksheet)
     Worksheet *worksheet;
{
  int i;

  for (i = 0; i < worksheet->nbchart; i++)
    {
      ActiveGraph = worksheet->Graphs[i];
      plot (0);
    }

  return 0;
}

int
worksheet_f (worksheet, x, y)
     Worksheet *worksheet;
     int x, y;
{

  return 0;
}

Cell *
worksheet_gcell (worksheet, r, c, new)
     Worksheet *worksheet;
     int r, c, new;
{
  int i;
  Cell *cell;
  Cell *ret;
  cell = worksheet->defcell;

  if (worksheet == NULL)
    return NULL;

  if (r < 0 || r > NBLIN)
    return worksheet->defcell;
  if (c < 0 || c > NBCOL)
    return worksheet->defcell;

  if (new < 0)
    {
      ret = NULL, new = 0;
    }
  else
    ret = worksheet->defcell;

  if (r < 0 || c < 0 || r > worksheet->nlin || c > worksheet->ncol)
    return ret;

  if (r > worksheet->mrow)
    {
      if (!new)
	return ret;

      worksheet->matrix = (Cell ***) absrealloc (worksheet->matrix, sizeof (Cell **) * (r + 1), "worksheet_gcell:worksheet->matrix ");
      worksheet->mcol = (int *) absrealloc (worksheet->mcol, sizeof (int) * (r + 1), "worksheet_gcell:worksheet->mcol ");

      for (i = worksheet->mrow + 1; i <= r; i++)
	{
	  worksheet->mcol[i] = 0;
	  worksheet->matrix[i] = NULL;
	}

      worksheet->matrix[r] =
	(Cell **) absrealloc (worksheet->matrix[r], sizeof (Cell *) * (c + 1), "worksheet_gcell:worksheet->matrix[r] ");
      for (i = 0; i <= c; i++)
	worksheet->matrix[r][i] = NULL;

      worksheet->mrow = r;
      if (c > worksheet->_mcol)
	worksheet->_mcol = c;
      worksheet->mcol[r] = c;

      worksheet->matrix[r][c] = newcell (r, c);
      return worksheet->matrix[r][c];
    }
  else if (c > worksheet->mcol[r])
    {
      if (!new)
	return ret;

      worksheet->matrix[r] =
	(Cell **) absrealloc (worksheet->matrix[r], sizeof (Cell *) * (c + 1), "worksheet_gcell:worksheet->matrix[r] ");
      for (i = worksheet->mcol[r] + 1; i <= c; i++)
	worksheet->matrix[r][i] = NULL;
      worksheet->mcol[r] = c;
      if (c > worksheet->_mcol)
	worksheet->_mcol = c;
      worksheet->matrix[r][c] = newcell (r, c);
      return worksheet->matrix[r][c];
    }

  else if (worksheet->matrix[r][c] == NULL)
    {
      if (!new)
	return ret;
      worksheet->matrix[r][c] = newcell (r, c);
      return worksheet->matrix[r][c];
    }

  return worksheet->matrix[r][c];

}

int
worksheet_delcell (worksheet, r, c)
     Worksheet *worksheet;
     int r, c;
{
  Cell *cell = worksheet_gcell (worksheet, r, c, -1);
  if (cell == NULL)
    return 1;

  if (cell->familly->npar > 0)
    cell_isnew (cell);

  if (cell->familly->nchi > 0)
    cell_setformula (cell, "");
  else
    {
      worksheet->matrix[r][c] = NULL;
      freecell (cell);
    }
  return 0;
}

Graph *
worksheet_nchart (worksheet)
     Worksheet *worksheet;
{
  worksheet->Graphs = (Graph **) absrealloc (worksheet->Graphs, sizeof (Graph *) * (worksheet->nbchart + 1), "worksheet_nchart:worksheet->Graphs ");
  worksheet->Graphs[worksheet->nbchart] = newchart ();
  worksheet->activechart = worksheet->Graphs[worksheet->nbchart];
  worksheet->nbchart++;
  return worksheet->Graphs[worksheet->nbchart - 1];
}

Drawing *
worksheet_ndrawing (worksheet)
     Worksheet *worksheet;
{
  worksheet->Drawings = (Drawing **) absrealloc (worksheet->Drawings, sizeof (Drawing *) * (worksheet->nbdrawing + 1), "worksheet_ndrawing:worksheet->Drawings ");
  worksheet->Drawings[worksheet->nbdrawing] = newdrawing ();
  worksheet->activedrawing = worksheet->Drawings[worksheet->nbdrawing];
  worksheet->nbdrawing++;
  return worksheet->Drawings[worksheet->nbdrawing - 1];
}

Button *
worksheet_nbutton (worksheet)
     Worksheet *worksheet;
{
  worksheet->Buttons = (Button **) absrealloc (worksheet->Buttons, sizeof (Button *) * (worksheet->nbbutton + 1), "worksheet_nbutton:worksheet->Buttons ");
  worksheet->Buttons[worksheet->nbbutton] = newbutton ();
  worksheet->activebutton = worksheet->Buttons[worksheet->nbbutton];
  worksheet->nbbutton++;
  return worksheet->Buttons[worksheet->nbbutton - 1];
}

extern int repaint_activesheet ();

Worksheet *
ActivateWorksheet (worksheet)
     Worksheet *worksheet;
{
  int i;

  if (worksheet == NULL)
    return NULL;

  if (ActiveWorksheet != worksheet && ActiveWorksheet != NULL)
    {
      for (i = 0; i < ActiveWorksheet->nbchart; i++)
	deletechartpixmap (ActiveWorksheet->Graphs[i]);
    }

  ActiveWorksheet = worksheet;
  ActiveWorkbook->activeworksheet = worksheet;

  ActiveCell = worksheet->activecell;
  ActiveGraph = worksheet->activechart;
  ActiveDrawing = worksheet->activedrawing;
  ActiveButton = worksheet->activebutton;
  ActiveSelection = worksheet->selection;

  for (i = 0; i < worksheet->nbchart; i++)
    {
      createchartpixmap (worksheet->Graphs[i]);
      newplot (i, worksheet->Graphs[i]);
    }

  cb_setzm (worksheet->zm);
  setzmlabel (worksheet->zm);
  setscrollvlimit (1, NBLIN);
  setscrollhlimit (1, NBCOL);
  setscrollv ((double) worksheet->ll);
  setscrollh ((double) worksheet->cc);

  repaint_activesheet ();

  return ActiveWorksheet;
}

Chart *
worksheet_gchart (worksheet, n)
     Worksheet *worksheet;
     int n;
{
  if (n > 0 && n < worksheet->nbchart)
    return worksheet->Graphs[n];
  return NULL;
}

Graph *
worksheet_addchart (worksheet, args, n)
     Worksheet *worksheet;
     int *args;
     int n;
{
  worksheet->Graphs = (Graph **) absrealloc (worksheet->Graphs, sizeof (Graph *) * (worksheet->nbchart + 1), "worksheet_nchart:worksheet->Graphs ");
  worksheet->Graphs[worksheet->nbchart] = newchart ();
  worksheet->activechart = worksheet->Graphs[worksheet->nbchart];
  worksheet->nbchart++;

  if (n == 4)
    {
      worksheet->activechart->x = args[0];
      worksheet->activechart->y = args[1];
      worksheet->activechart->w = args[2];
      worksheet->activechart->h = args[3];
    }
  else
    {
      worksheet->activechart->x = 30;
      worksheet->activechart->y = 30;
      worksheet->activechart->w = 80;
      worksheet->activechart->h = 50;
    }

  createchartpixmap (worksheet->activechart);
  ActiveGraph = worksheet->activechart;
  return ActiveGraph;
}

int
worksheet_addshape (worksheet, args, n)
     Worksheet *worksheet;
     int *args;
     int n;
{
  worksheet->Drawings = (Drawing **) absrealloc (worksheet->Drawings, sizeof (Drawing *) * (worksheet->nbdrawing + 1)
				,"worksheet_addshape:worksheet->Drawings ");
  worksheet->Drawings[worksheet->nbdrawing] = (Drawing *) newdrawing ();
  worksheet->activedrawing = worksheet->Drawings[worksheet->nbdrawing];
  worksheet->nbdrawing++;

  if (n == 5)
    {
      worksheet->activedrawing->type = args[0];
      worksheet->activedrawing->x1 = args[1];
      worksheet->activedrawing->y1 = args[2];
      worksheet->activedrawing->x2 = args[3];
      worksheet->activedrawing->y2 = args[4];
    }
  if (n == 4)
    {
      worksheet->activedrawing->type = 0;
      worksheet->activedrawing->x1 = args[0];
      worksheet->activedrawing->y1 = args[1];
      worksheet->activedrawing->x2 = args[2];
      worksheet->activedrawing->y2 = args[3];
    }

  ActiveDrawing = worksheet->activedrawing;
  return 0;
}

Button *
worksheet_addbutton (worksheet, args, n)
     Worksheet *worksheet;
     int *args;
     int n;
{
  worksheet->Buttons = (Button **) absrealloc (worksheet->Buttons, sizeof (Button *) * (worksheet->nbbutton + 1), "worksheet_addbutton:worksheet->Buttons ");
  worksheet->Buttons[worksheet->nbbutton] = (Button *) newbutton ();
  worksheet->activebutton = worksheet->Buttons[worksheet->nbbutton];
  worksheet->nbbutton++;

  worksheet->activebutton->x1 = args[0];
  worksheet->activebutton->y1 = args[1];
  worksheet->activebutton->x2 = args[2] + args[0];
  worksheet->activebutton->y2 = args[3] + args[1];

  ActiveButton = worksheet->activebutton;
  return ActiveButton;
}

int
worksheet_setcw (worksheet, col, w)
     Worksheet *worksheet;
     int col;
     double w;
{
  int i;
  int dw;
  w = w * 7.2;
  dw = w - (worksheet->cw[col + 1] - worksheet->cw[col]);
  for (i = col + 1; i < worksheet->ncol; i++)
    worksheet->cw[i] += dw;
  return 0;
}

int
worksheet_setrh (worksheet, lin, h)
     Worksheet *worksheet;
     int lin;
     double h;
{
  int i;
  int dh = h - (worksheet->lh[lin + 1] - worksheet->lh[lin]);
  for (i = lin + 1; i < worksheet->nlin; i++)
    worksheet->lh[i] += dh;
  return 0;
}

int
worksheet_setname (worksheet, name)
     Worksheet *worksheet;
     char *name;
{
  int len = 0;
  int i, j, k;
  Cell *cell;
  char oldname[30];
  if (name == NULL)
    return -1;
  len = strlen (name);
  if (len < 1)
    return -1;
  if (len > 30)
    {
      name[29] = '\0';
      len = 30;
    }

  strcpy (oldname, worksheet->Name);
  strcpy (worksheet->Name, name);
  worksheet->Name[29] = '\0';
  rebuild_thumb ();

  for (i = 1; i <= worksheet->mrow; i++)
    for (j = 1; j <= worksheet->mcol[i]; j++)
      {
	cell = worksheet->matrix[i][j];
	if (cell != NULL)
	  {
	    for (k = 0; k < cell->familly->nchi; k++)
	      {
		fprintf (stderr, "cell %d %d has child %d %d\n", cell->r, cell->c, cell->familly->chilst[k]->r, cell->familly->chilst[k]->c);
		cell_chgwksname (cell->familly->chilst[k], oldname, name);
	      }
	  }
      }
  return 0;
}

int
worksheet_calculate (Worksheet * worksheet)
{
  int i, j;
  for (i = 1; i <= worksheet->mrow; i++)
    for (j = 1; j <= worksheet->mcol[i]; j++)
      {
	if (worksheet->matrix[i][j] != NULL)
	  cell_calculate (worksheet->matrix[i][j]);
      }
  if (worksheet == ActiveWorksheet)
    repaint_activesheet ();
  return 0;
}

int
worksheet_printout (Worksheet * worksheet)
{

  return 0;
}
