/********************************************************************
This file is part of the abs 0.6a distribution.  abs is a spreadsheet
with graphical user interface.

Copyright (C) 1998-1999  Andr Bertin (Andre.Bertin@ping.be) 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version if in the same spirit as version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Concact: Andre.Bertin@ping.be
         http://www.ping.be/bertin/abs.shtml

*********************************************************************/

#ifndef WORKSHEET_H
#define WORKSHEET_H
#include "cell.h"
#include "chart.h"
#include "draw.h"
#include "button.h"
#include "selection.h"

#include "stdio.h"

#define NBCOL  256
#define NBLIN  1000
#define WCOL     84
#define HLIN     20


struct _Worksheet
{
 Cell *defcell;
 Graph **Graphs;
 Drawing **Drawings;
 Button **Buttons;
 char Name[32];

 Cell *activecell;
 Graph *activechart;
 Drawing *activedrawing;
 Button *activebutton;

 int nbcell, nbchart, nbdrawing, nbbutton;
 int zm;
 int objn;
 int nlin, ncol;
 int cw[NBCOL + 1];
 int lh[NBLIN + 1];
 int cc, ll;
 Selection* selection;
 Cell* ** matrix;
 int mrow;
 int * mcol;
 int _mcol;
};

typedef struct _Worksheet Worksheet;

extern Worksheet *ActiveWorksheet;

extern Worksheet *newworksheet();
extern Worksheet *newworksheet1(int nbcell, int nbchart, int nbdrawing);
extern int freeworksheet();

extern int worksheet_w(Worksheet *w, FILE * fp);
extern int worksheet_update(Worksheet *worksheet);

extern int worksheet_f(Worksheet *worksheet, int x, int y);
extern Cell *worksheet_gcell(Worksheet *worksheet, int r, int c, int new);
extern int worksheet_delcell(Worksheet *worksheet, int r, int c);
extern Graph *worksheet_nchart(Worksheet *worksheet);
extern Drawing *worksheet_ndrawing(Worksheet *worksheet);
extern Button *worksheet_nbutton(Worksheet *worksheet);

extern Worksheet * ActivateWorksheet(Worksheet *worksheet);

extern Chart * worksheet_gchart(Worksheet *worksheet, int n);
extern Chart * worksheet_addchart(Worksheet *worksheet, int *args, int n);
extern int worksheet_addshape(Worksheet *worksheet, int *args, int n);
extern Button *worksheet_addbutton(Worksheet *worksheet, int *args, int n);
extern int worksheet_setcw (Worksheet *worksheet, int col, double w);
extern int worksheet_setrh (Worksheet *worksheet, int lin, double h);
extern int worksheet_setname (Worksheet *worksheet, char* name);
extern int worksheet_calculate (Worksheet *worksheet);
extern int worksheet_printout (Worksheet *worksheet);

#endif				/* WORKSHEET_H */
